from typing import Optional, Union
from .statsig_ai_base import StatsigAIInstance
from .statsig_ai_base import StatsigCreateConfig, StatsigAttachConfig


class StatsigAI(StatsigAIInstance):
    _statsig_ai_shared_instance: Optional[StatsigAIInstance] = None

    @classmethod
    def shared(cls) -> StatsigAIInstance:
        if not cls.has_shared_instance() or cls._statsig_ai_shared_instance is None:
            return create_statsig_ai_error_instance(
                "StatsigAI.shared() called, but no instance has been set with StatsigAI.new_shared(...)"
            )
        return cls._statsig_ai_shared_instance

    @classmethod
    def new_shared(
        cls,
        statsig_source: Union[StatsigCreateConfig, StatsigAttachConfig],
    ) -> StatsigAIInstance:
        if cls.has_shared_instance():
            return create_statsig_ai_error_instance(
                "StatsigAI shared instance already exists. "
                "Call StatsigAI.remove_shared() before creating a new instance."
            )

        cls._statsig_ai_shared_instance = StatsigAIInstance(statsig_source=statsig_source)
        return cls._statsig_ai_shared_instance

    @classmethod
    def remove_shared(cls) -> None:
        cls._statsig_ai_shared_instance = None

    @classmethod
    def has_shared_instance(cls) -> bool:
        return cls._statsig_ai_shared_instance is not None


def create_statsig_ai_error_instance(message: str) -> StatsigAIInstance:
    print("Error: ", message)
    return StatsigAIInstance(
        statsig_source=StatsigCreateConfig(sdk_key="__STATSIG_ERROR_SDK_KEY__")
    )



from dataclasses import dataclass


@dataclass
class GenAICaptureOptions:
    capture_all: bool = False
    capture_input_messages: bool = False
    capture_output_messages: bool = False
    capture_system_instructions: bool = False
    capture_tool_definitions: bool = False


class WrapOpenAIOptions:
    def __init__(self, gen_ai_capture_options: GenAICaptureOptions = GenAICaptureOptions()):
        self.gen_ai_capture_options = gen_ai_capture_options



import json
import re
from typing import Any, Dict, List, TypedDict
from statsig_python_core import DynamicConfig


PromptParams = Dict[str, Any]


class PromptMessage(TypedDict):
    role: str
    content: str


class PromptVersion:
    def __init__(self, prompt_variant: DynamicConfig):
        self.name: str = prompt_variant.get_string("name", "")
        self._type: str = prompt_variant.get_string("type", "")
        self._ai_config_name: str = prompt_variant.get_string("aiConfigName", "")

        parts = prompt_variant.name.split(":")
        self._id: str = parts[1] if len(parts) > 1 else ""

        self._prompt_variant = prompt_variant

    def get_name(self) -> str:
        return self.name

    def get_id(self) -> str:
        return self._id

    def get_type(self) -> str:
        return self._type

    def get_prompt_name(self) -> str:
        return self._ai_config_name

    def get_temperature(self, fallback: float = 0) -> float:
        return self._prompt_variant.get_float("temperature", fallback)

    def get_max_tokens(self, fallback: int = 0) -> int:
        return self._prompt_variant.get_integer("maxTokens", fallback)

    def get_top_p(self, fallback: float = 0) -> float:
        return self._prompt_variant.get_float("topP", fallback)

    def get_frequency_penalty(self, fallback: float = 0) -> float:
        return self._prompt_variant.get_float("frequencyPenalty", fallback)

    def get_presence_penalty(self, fallback: float = 0) -> float:
        return self._prompt_variant.get_float("presencePenalty", fallback)

    def get_provider(self, fallback: str = "") -> str:
        return self._prompt_variant.get_string("provider", fallback)

    def get_model(self, fallback: str = "") -> str:
        return self._prompt_variant.get_string("model", fallback)

    # pylint: disable=dangerous-default-value
    def get_workflow_body(self, fallback: Dict[str, Any] = {}) -> Dict[str, Any]:
        workflow_body_json = self._prompt_variant.get_object_json(
            "workflowBody", json.dumps(fallback)
        )
        return json.loads(workflow_body_json)

    def get_eval_model(self, fallback: str = "") -> str:
        return self._prompt_variant.get_string("evalModel", fallback)

    def get_string(self, key: str, fallback: str) -> str:
        return self._prompt_variant.get_string(key, fallback)

    def get_integer(self, key: str, fallback: int) -> int:
        return self._prompt_variant.get_integer(key, fallback)

    def get_float(self, key: str, fallback: float) -> float:
        return self._prompt_variant.get_float(key, fallback)

    def get_boolean(self, key: str, fallback: bool) -> bool:
        return self._prompt_variant.get_bool(key, fallback)

    def get_array(self, key: str, fallback: list) -> list:
        array_json = self._prompt_variant.get_array_json(key, json.dumps(fallback))
        return json.loads(array_json)

    def get_object(self, key: str, fallback: dict) -> dict:
        object_json = self._prompt_variant.get_object_json(key, json.dumps(fallback))
        return json.loads(object_json)

    def get_prompt_messages(self, params: PromptParams) -> List[PromptMessage]:
        prompts_array_json = self._prompt_variant.get_array_json(
            "prompts", json.dumps([{"role": "system", "content": ""}])
        )
        prompts = json.loads(prompts_array_json)
        regex = re.compile(r"{{\s*([^}]+)\s*}}")

        def replace_placeholders(content: str) -> str:
            def replacer(match):
                path = match.group(1).strip()
                value = resolve_path(params, path)
                return str(value) if value is not None else f"{{{{{path}}}}}"

            return regex.sub(replacer, content)

        return [
            PromptMessage(role=p["role"], content=replace_placeholders(p["content"]))
            for p in prompts
        ]


def resolve_path(obj: Any, path: str) -> Any:
    """Safely resolves nested object paths, supporting array indices like `a.b[0].c`."""
    parts = path.replace("[", ".").replace("]", "").split(".")
    current = obj
    for key in parts:
        if current is None or key == "":
            return None
        if isinstance(current, dict):
            current = current.get(key)
        elif isinstance(current, list):
            try:
                idx = int(key)
                current = current[idx]
            except (ValueError, IndexError):
                return None
        else:
            return None
    return current



from opentelemetry.sdk.trace.export import BatchSpanProcessor


class StatsigSpanProcessor(BatchSpanProcessor):
    """
    A Statsig wrapper around the OpenTelemetry BatchSpanProcessor.

    Currently identical to BatchSpanProcessor, but provides a named subclass
    for future customization.
    """



from statsig_python_core import StatsigOptions, StatsigUser
from pytest_httpserver import HTTPServer
from statsig_ai import StatsigAI
from statsig_ai.statsig_ai_base import StatsigCreateConfig
from mock_scrapi import MockScrapi
from utils import get_test_data_resource
import pytest


@pytest.fixture
def statsig_setup(httpserver: HTTPServer):
    mock_scrapi = MockScrapi(httpserver)
    dcs_content = get_test_data_resource("eval_proj_dcs_targeting.json")

    mock_scrapi.stub(
        "/v2/download_config_specs/secret-key.json", response=dcs_content, method="GET"
    )
    mock_scrapi.stub("/v1/log_event", response='{"success": true}', method="POST")

    options = StatsigOptions(
        specs_url=mock_scrapi.url_for_endpoint("/v2/download_config_specs"),
        log_event_url=mock_scrapi.url_for_endpoint("/v1/log_event"),
    )

    yield options, mock_scrapi


def test_get_prompt_with_targeting(statsig_setup):
    options, _ = statsig_setup
    statsig_ai = StatsigAI(
        statsig_source=StatsigCreateConfig(sdk_key="secret-key", statsig_options=options)
    )
    statsig_ai.initialize()
    prompt = statsig_ai.get_prompt(StatsigUser("test-user"), "test-prompt-1")
    statsig_ai.shutdown()
    live_version = prompt.get_live()
    assert live_version is not None
    assert live_version.get_id() == "2QGncLi0YSYj9zavJ825qB"
    assert live_version.get_name() == "Version 1"
    assert live_version.get_temperature() == 0
    assert live_version.get_max_tokens() == 1000
    assert live_version.get_top_p() == 1
    assert live_version.get_frequency_penalty() == 0
    assert live_version.get_presence_penalty() == 0
    assert live_version.get_provider() == "openai"
    assert live_version.get_model() == "gpt-4o"
    assert live_version.get_prompt_name() == "test-prompt-2"


def test_get_prompt_with_nested_targeting(statsig_setup):
    options, _ = statsig_setup
    statsig_ai = StatsigAI(
        statsig_source=StatsigCreateConfig(sdk_key="secret-key", statsig_options=options)
    )
    statsig_ai.initialize()
    prompt = statsig_ai.get_prompt(StatsigUser("test-user-1"), "test-prompt-1")
    statsig_ai.shutdown()
    live_version = prompt.get_live()
    assert live_version is not None
    assert live_version.get_id() == "36qqmfh8wHcKzjmX2My2RQ"
    assert live_version.get_name() == "Version 1"
    assert live_version.get_temperature() == 2
    assert live_version.get_max_tokens() == 1000
    assert live_version.get_top_p() == 1
    assert live_version.get_frequency_penalty() == 0
    assert live_version.get_presence_penalty() == 0
    assert live_version.get_provider() == "openai"
    assert live_version.get_model() == "gpt-4.1"
    assert live_version.get_prompt_name() == "test-prompt-3"


def test_get_prompt_with_circular_targeting_without_crashing(statsig_setup, capsys):
    options, _ = statsig_setup
    statsig_ai = StatsigAI(
        statsig_source=StatsigCreateConfig(sdk_key="secret-key", statsig_options=options)
    )
    statsig_ai.initialize()
    prompt = statsig_ai.get_prompt(StatsigUser("test-user-circular"), "test-prompt-circular-a")
    statsig_ai.shutdown()
    captured = capsys.readouterr()
    assert (
        "[Statsig] Max targeting depth (300) reached while resolving prompt: test-prompt-circular-a."
        in captured.out
    )
    assert (
        'Possible circular reference starting from "prompt:test-prompt-circular-a".' in captured.out
    )
    live_version = prompt.get_live()
    assert live_version is not None
    assert live_version.get_name() == "Circular A Version"
    assert live_version.get_temperature() == 1
    assert live_version.get_max_tokens() == 500
    assert live_version.get_top_p() == 1
    assert live_version.get_prompt_name() == "test-prompt-circular-a"



from dataclasses import fields, is_dataclass
from typing import Any, TypeVar, cast
from datetime import datetime, date
from enum import Enum

T = TypeVar("T", bound="SerializableDataClass")


class SerializableDataClass:
    """
    Base class for dataclasses that provides easy serialization

    Usage:
        @dataclass
        class MyClass(SerializableDataclass):
            name: str
            value: int
    """

    def to_dict(self) -> dict[str, Any]:
        """Convert dataclass instance to dictionary, handling nested structures. Exclude None values"""
        result = {}
        for field in fields(cast(Any, self)):
            value = getattr(self, field.name)
            serialized_value = self._serialize_value(value)
            if serialized_value is not None:
                result[field.name] = serialized_value
        return result

    @staticmethod
    def _serialize_value(value: Any) -> Any:
        """Serialize a single value, handling various types."""
        if value is None:
            return None
        if isinstance(value, (str, int, float, bool)):
            return value
        if isinstance(value, Enum):
            return value.value
        if isinstance(value, (datetime, date)):
            return value.isoformat()
        if is_dataclass(value) and hasattr(value, "to_dict"):
            return value.to_dict()  # type: ignore
        if isinstance(value, dict):
            return {k: SerializableDataClass._serialize_value(v) for k, v in value.items()}
        if isinstance(value, (list, tuple, set)):
            return [SerializableDataClass._serialize_value(item) for item in value]
        # Fallback for other types
        return str(value)



from opentelemetry import trace
from opentelemetry.trace import TracerProvider


class OtelSingleton:
    _instance = None

    def __init__(self, tracer_provider: TracerProvider):
        self._tracer_provider = tracer_provider

    @classmethod
    def instantiate(cls, tracer_provider: TracerProvider):
        """
        Create or return an existing OtelSingleton instance.
        """
        if cls._instance is not None:
            print(
                "[Statsig otel] OtelSingleton instance already exists. "
                "Returning the existing instance."
            )
            return cls._instance
        cls._instance = cls(tracer_provider)
        return cls._instance

    @classmethod
    def get_instance(cls):
        if cls._instance is not None:
            return cls._instance
        return NoopOtelSingleton.get_instance()

    def get_tracer_provider(self) -> TracerProvider:
        return self._tracer_provider

    @classmethod
    def get_tracer_provider_static(cls) -> TracerProvider:
        return cls.get_instance().get_tracer_provider()


class NoopOtelSingleton(OtelSingleton):
    def __init__(self):
        super().__init__(trace.get_tracer_provider())

    @classmethod
    def get_instance(cls):
        print(
            "[Statsig otel] NoopOtelSingleton instance is being used. "
            "OtelSingleton has not been properly instantiated."
        )
        return cls()



import pytest
from pytest_httpserver import HTTPServer
from statsig_ai import Eval, EvalScorerArgs, EvalDataRecord, EvalHook
from mock_scrapi import MockScrapi
import math


@pytest.fixture
def eval_setup(httpserver: HTTPServer, monkeypatch):
    from werkzeug import Response
    import re

    mock_scrapi = MockScrapi(httpserver)

    def eval_handler(request):
        if re.match(r"/console/v1/evals/send_results/.+", request.path):
            data = request.get_data()
            import json

            req_json = json.loads(data)
            mock_scrapi.eval_payloads.append(req_json)
            return Response('{"success": true}', status=200)
        return Response("Not Found", status=404)

    httpserver.expect_request(
        re.compile(r"/console/v1/evals/send_results/.+"), method="POST"
    ).respond_with_handler(eval_handler)

    monkeypatch.setenv("STATSIG_API_KEY", "test-api-key")

    import requests

    original_post = requests.post

    def intercepted_post(url, *args, **kwargs):
        if "api.statsig.com/console/v1/evals/send_results/" in url:
            eval_name = url.split("/send_results/")[-1]
            mock_url = (
                f"{mock_scrapi.url_for_endpoint('/console/v1/evals/send_results/')}{eval_name}"
            )
            return original_post(mock_url, *args, **kwargs)
        return original_post(url, *args, **kwargs)

    monkeypatch.setattr(requests, "post", intercepted_post)

    yield mock_scrapi


def test_eval_basic_string(eval_setup):
    mock_scrapi = eval_setup

    def scorer(args: EvalScorerArgs[str, str]) -> float:
        return 1.0 if args.output == args.expected else 0.0

    result = Eval(
        "test_basic_string",
        data=[
            {"input": "world", "expected": "Hello world"},
            {"input": "test", "expected": "Hello test"},
        ],
        task=lambda input: f"Hello {input}",
        scorer=scorer,
    )

    assert len(result.results) == 2
    assert result.metadata.error == False
    assert result.results[0].input == "world"
    assert result.results[0].output == "Hello world"
    assert result.results[0].expected == "Hello world"
    assert result.results[0].scores["Grader"] == 1.0

    payloads = mock_scrapi.get_eval_payloads()
    assert len(payloads) == 1
    payload = payloads[0]
    assert "results" in payload
    assert len(payload["results"]) == 2
    assert payload["results"][0]["input"] == "world"
    assert payload["results"][0]["output"] == "Hello world"
    assert payload["results"][0]["expected"] == "Hello world"
    assert payload["results"][0]["scores"]["Grader"] == 1.0


def test_eval_task_with_hook(eval_setup):
    mock_scrapi = eval_setup

    def task_with_hook(input: str, hook: EvalHook) -> str:
        prefix = hook.parameters.get("prefix", "Hello") if hook.parameters else "Hello"
        return f"{prefix} {input}"

    def scorer(args: EvalScorerArgs[str, str]) -> float:
        return 1.0 if args.output == args.expected else 0.0

    result = Eval(
        "test_task_with_hook",
        data=[
            {"input": "world", "expected": "Hi world"},
        ],
        task=task_with_hook,
        scorer=scorer,
        parameters={"prefix": "Hi"},
    )

    assert result.results[0].output == "Hi world"
    assert result.results[0].scores["Grader"] == 1.0

    payloads = mock_scrapi.get_eval_payloads()
    assert len(payloads) == 1
    assert payloads[0]["results"][0]["output"] == "Hi world"
    assert payloads[0]["results"][0]["scores"]["Grader"] == 1.0


def test_eval_multiple_scorers(eval_setup):
    mock_scrapi = eval_setup

    def exact_match_scorer(args: EvalScorerArgs[str, str]) -> bool:
        return args.output == args.expected

    def length_scorer(args: EvalScorerArgs[str, str]) -> int:
        return len(args.output) > 5

    result = Eval(
        "test_multiple_scorers",
        data=[
            {"input": "world", "expected": "Hello world"},
        ],
        task=lambda input: f"Hello {input}",
        scorer={
            "exact_match": exact_match_scorer,
            "length": length_scorer,
        },
    )

    assert "exact_match" in result.results[0].scores
    assert "length" in result.results[0].scores
    assert result.results[0].scores["exact_match"] == 1.0
    assert result.results[0].scores["length"] == 1.0

    payloads = mock_scrapi.get_eval_payloads()
    assert len(payloads) == 1
    assert payloads[0]["results"][0]["scores"]["exact_match"] == 1.0
    assert payloads[0]["results"][0]["scores"]["length"] == 1.0


def test_eval_async_task(eval_setup):
    mock_scrapi = eval_setup

    async def async_task(input: str) -> str:
        return f"Async {input}"

    def scorer(args: EvalScorerArgs[str, str]) -> float:
        return 1.0 if "Async" in args.output else 0.0

    result = Eval(
        "test_async_task",
        data=[
            {"input": "world"},
        ],
        task=async_task,
        scorer=scorer,
    )

    assert result.results[0].output == "Async world"
    assert result.results[0].scores["Grader"] == 1.0

    payloads = mock_scrapi.get_eval_payloads()
    assert len(payloads) == 1
    assert payloads[0]["results"][0]["output"] == "Async world"
    assert payloads[0]["results"][0]["scores"]["Grader"] == 1.0


def test_eval_async_scorer(eval_setup):
    mock_scrapi = eval_setup

    async def async_scorer(args: EvalScorerArgs[str, str]) -> float:
        return 1.0 if args.output == args.expected else 0.0

    result = Eval(
        "test_async_scorer",
        data=[
            {"input": "world", "expected": "Hello world"},
        ],
        task=lambda input: f"Hello {input}",
        scorer=async_scorer,
    )

    assert result.results[0].scores["Grader"] == 1.0

    payloads = mock_scrapi.get_eval_payloads()
    assert len(payloads) == 1
    assert payloads[0]["results"][0]["scores"]["Grader"] == 1.0


def test_eval_dict_input(eval_setup):
    mock_scrapi = eval_setup

    def scorer(args: EvalScorerArgs[dict, str]) -> float:
        return 1.0 if len(args.output) > 8 else 0.0

    result = Eval(
        "test_dict_input",
        data=[
            {"input": {"name": "Alice", "age": 30}, "expected": "Hello Alice"},
        ],
        task=lambda input: f"Hello {input['name']}",
        scorer=scorer,
    )

    assert result.results[0].input == {"name": "Alice", "age": 30}
    assert result.results[0].output == "Hello Alice"
    assert result.results[0].scores["Grader"] == 1.0

    payloads = mock_scrapi.get_eval_payloads()
    assert len(payloads) == 1
    assert payloads[0]["results"][0]["input"] == {"name": "Alice", "age": 30}
    assert payloads[0]["results"][0]["output"] == "Hello Alice"
    assert payloads[0]["results"][0]["scores"]["Grader"] == 1.0


def test_eval_int_input(eval_setup):
    mock_scrapi = eval_setup

    def scorer(args: EvalScorerArgs[int, int]) -> bool:
        return args.output == args.expected

    result = Eval(
        "test_int_input",
        data=[
            {"input": 5, "expected": 10},
            {"input": 3, "expected": 6},
        ],
        task=lambda input: input * 2,
        scorer=scorer,
    )

    assert result.results[0].input == 5
    assert result.results[0].output == 10
    assert result.results[0].scores["Grader"] == 1.0

    payloads = mock_scrapi.get_eval_payloads()
    assert len(payloads) == 1
    assert payloads[0]["results"][0]["input"] == 5
    assert payloads[0]["results"][0]["output"] == 10
    assert payloads[0]["results"][0]["scores"]["Grader"] == 1.0


def test_eval_dataclass_records(eval_setup):
    mock_scrapi = eval_setup

    def scorer(args: EvalScorerArgs[str, str]) -> float:
        return 1.0 if len(args.output) > 8 else 0.0

    result = Eval(
        "test_dataclass_records",
        data=[
            EvalDataRecord(input="world", expected="Hello world"),
            EvalDataRecord(input="test", expected="Hello test"),
        ],
        task=lambda input: f"Hello {input}",
        scorer=scorer,
    )

    assert len(result.results) == 2
    assert result.results[0].scores["Grader"] == 1.0
    assert result.results[0].output == "Hello world"

    payloads = mock_scrapi.get_eval_payloads()
    assert len(payloads) == 1
    assert len(payloads[0]["results"]) == 2
    assert payloads[0]["results"][0]["scores"]["Grader"] == 1.0
    assert payloads[0]["results"][0]["output"] == "Hello world"


def test_eval_iterator_data(eval_setup):
    mock_scrapi = eval_setup

    def data_generator():
        yield {"input": "world", "expected": "Hello world"}
        yield {"input": "test", "expected": "Hello test"}

    def scorer(args: EvalScorerArgs[str, str]) -> float:
        return 1.0

    result = Eval(
        "test_iterator_data",
        data=data_generator(),
        task=lambda input: f"Hello {input}",
        scorer=scorer,
    )

    assert len(result.results) == 2
    assert result.results[0].input == "world"
    assert result.results[0].output == "Hello world"
    assert result.results[0].scores["Grader"] == 1.0
    assert result.results[1].input == "test"
    assert result.results[1].output == "Hello test"
    assert result.results[1].scores["Grader"] == 1.0

    payloads = mock_scrapi.get_eval_payloads()
    assert len(payloads) == 1
    assert len(payloads[0]["results"]) == 2
    assert payloads[0]["results"][0]["input"] == "world"
    assert payloads[0]["results"][0]["output"] == "Hello world"
    assert payloads[0]["results"][0]["scores"]["Grader"] == 1.0


def test_eval_callable_data(eval_setup):
    mock_scrapi = eval_setup

    def data_func():
        return [
            {"input": "world", "expected": "Hello world"},
            {"input": "test", "expected": "Hello test"},
        ]

    def scorer(args: EvalScorerArgs[str, str]) -> float:
        return 1.0

    result = Eval(
        "test_callable_data",
        data=data_func,
        task=lambda input: f"Hello {input}",
        scorer=scorer,
    )

    assert len(result.results) == 2
    assert result.results[0].input == "world"
    assert result.results[0].output == "Hello world"
    assert result.results[0].scores["Grader"] == 1.0

    payloads = mock_scrapi.get_eval_payloads()
    assert len(payloads) == 1
    assert len(payloads[0]["results"]) == 2
    assert payloads[0]["results"][0]["input"] == "world"
    assert payloads[0]["results"][0]["output"] == "Hello world"
    assert payloads[0]["results"][0]["scores"]["Grader"] == 1.0


def test_eval_async_iterator_data(eval_setup):
    mock_scrapi = eval_setup

    async def async_data_generator():
        yield {"input": "world", "expected": "Hello world"}
        yield {"input": "test", "expected": "Hello test"}

    def scorer(args: EvalScorerArgs[str, str]) -> float:
        return 1.0

    result = Eval(
        "test_async_iterator_data",
        data=async_data_generator(),
        task=lambda input: f"Hello {input}",
        scorer=scorer,
    )

    assert len(result.results) == 2
    assert result.results[0].input == "world"
    assert result.results[0].output == "Hello world"
    assert result.results[0].scores["Grader"] == 1.0

    payloads = mock_scrapi.get_eval_payloads()
    assert len(payloads) == 1
    assert len(payloads[0]["results"]) == 2
    assert payloads[0]["results"][0]["input"] == "world"
    assert payloads[0]["results"][0]["output"] == "Hello world"
    assert payloads[0]["results"][0]["scores"]["Grader"] == 1.0


def test_eval_with_categories(eval_setup):
    mock_scrapi = eval_setup

    def task_with_hook(input: str, hook: EvalHook) -> str:
        return f"Hello {input}"

    def scorer(args: EvalScorerArgs[str, str]) -> float:
        return 1.0

    result = Eval(
        "test_with_categories",
        data=[
            {"input": "world", "expected": "Hello world", "category": "greeting"},
            {"input": "test", "expected": "Hello test", "category": ["greeting", "test"]},
        ],
        task=task_with_hook,
        scorer=scorer,
    )

    assert result.results[0].category == "greeting"
    assert result.results[1].category == ["greeting", "test"]

    payloads = mock_scrapi.get_eval_payloads()
    assert len(payloads) == 1
    assert payloads[0]["results"][0]["category"] == "greeting"
    assert payloads[0]["results"][1]["category"] == ["greeting", "test"]


def test_eval_without_expected(eval_setup):
    mock_scrapi = eval_setup

    def scorer(args: EvalScorerArgs[str, str]) -> float:
        return 1.0 if len(args.output) > 8 else 0.0

    result = Eval(
        "test_without_expected",
        data=[
            {"input": "world"},
            {"input": "hi"},
        ],
        task=lambda input: f"Hello {input}",
        scorer=scorer,
    )

    assert result.results[0].expected is None
    assert result.results[0].scores["Grader"] == 1.0
    assert result.results[1].scores["Grader"] == 0.0

    payloads = mock_scrapi.get_eval_payloads()
    assert len(payloads) == 1
    assert payloads[0]["results"][0].get("expected") is None
    assert payloads[0]["results"][0]["scores"]["Grader"] == 1.0
    assert payloads[0]["results"][1]["scores"]["Grader"] == 0.0


def test_eval_with_run_name(eval_setup):
    mock_scrapi = eval_setup

    def scorer(args: EvalScorerArgs[str, str]) -> float:
        return 1.0

    result = Eval(
        "test_with_run_name",
        data=[
            {"input": "world", "expected": "Hello world"},
        ],
        task=lambda input: f"Hello {input}",
        scorer=scorer,
        eval_run_name="test_run_123",
    )

    assert result.results[0].input == "world"
    assert result.results[0].output == "Hello world"
    assert result.results[0].scores["Grader"] == 1.0

    payloads = mock_scrapi.get_eval_payloads()
    assert len(payloads) == 1
    assert payloads[0].get("name") == "test_run_123"
    assert payloads[0]["results"][0]["input"] == "world"
    assert payloads[0]["results"][0]["output"] == "Hello world"
    assert payloads[0]["results"][0]["scores"]["Grader"] == 1.0


def test_eval_task_error(eval_setup):
    mock_scrapi = eval_setup

    def failing_task(input: str) -> str:
        if input == "fail":
            raise ValueError("Task failed")
        return f"Hello {input}"

    def scorer(args: EvalScorerArgs[str, str]) -> float:
        return 1.0

    result = Eval(
        "test_task_error",
        data=[
            {"input": "world", "expected": "Hello world"},
            {"input": "fail", "expected": "Hello fail"},
        ],
        task=failing_task,
        scorer=scorer,
    )

    assert result.results[0].output == "Hello world"
    assert result.results[0].error == False
    assert result.results[1].error == True
    assert result.results[1].scores["Grader"] == 0.0
    assert result.metadata.error == True

    payloads = mock_scrapi.get_eval_payloads()
    assert len(payloads) == 1
    assert payloads[0]["results"][0]["output"] == "Hello world"
    assert payloads[0]["results"][0]["error"] == False
    assert payloads[0]["results"][1]["error"] == True
    assert result.results[1].scores["Grader"] == 0.0


def test_eval_scorer_error(eval_setup):
    mock_scrapi = eval_setup

    def failing_scorer(args: EvalScorerArgs[str, str]) -> float:
        if args.input == "fail":
            raise ValueError("Scorer failed")
        return 1.0

    result = Eval(
        "test_scorer_error",
        data=[
            {"input": "world", "expected": "Hello world"},
            {"input": "fail", "expected": "Hello fail"},
        ],
        task=lambda input: f"Hello {input}",
        scorer=failing_scorer,
    )

    assert result.results[0].scores["Grader"] == 1.0
    assert result.results[1].scores["Grader"] == 0.0
    assert result.results[1].error == False

    payloads = mock_scrapi.get_eval_payloads()
    assert len(payloads) == 1
    assert payloads[0]["results"][0]["scores"]["Grader"] == 1.0
    assert payloads[0]["results"][1]["scores"]["Grader"] == 0.0
    assert payloads[0]["results"][1]["error"] == False


def test_eval_boolean_scorer(eval_setup):
    mock_scrapi = eval_setup

    def bool_scorer(args: EvalScorerArgs[str, str]) -> bool:
        return args.output == args.expected

    result = Eval(
        "test_boolean_scorer",
        data=[
            {"input": "world", "expected": "Hello world"},
            {"input": "test", "expected": "Goodbye test"},
        ],
        task=lambda input: f"Hello {input}",
        scorer=bool_scorer,
    )

    assert result.results[0].scores["Grader"] == 1.0
    assert result.results[1].scores["Grader"] == 0.0

    payloads = mock_scrapi.get_eval_payloads()
    assert len(payloads) == 1
    assert payloads[0]["results"][0]["scores"]["Grader"] == 1.0
    assert payloads[0]["results"][1]["scores"]["Grader"] == 0.0


def test_eval_class_based_scorer(eval_setup):
    mock_scrapi = eval_setup

    class MyScorer:
        def __call__(self, args: EvalScorerArgs[str, str]) -> float:
            return 1.0 if args.output == args.expected else 0.0

    result = Eval(
        "test_class_scorer",
        data=[
            {"input": "world", "expected": "Hello world"},
        ],
        task=lambda input: f"Hello {input}",
        scorer=MyScorer,
    )

    assert result.results[0].scores["Grader"] == 1.0

    payloads = mock_scrapi.get_eval_payloads()
    assert len(payloads) == 1
    assert payloads[0]["results"][0]["scores"]["Grader"] == 1.0


def test_eval_with_parameters(eval_setup):
    mock_scrapi = eval_setup

    def task_with_params(input: str, hook: EvalHook) -> str:
        prefix = hook.parameters.get("prefix", "Hello")
        suffix = hook.parameters.get("suffix", "!")
        return f"{prefix} {input}{suffix}"

    def scorer(args: EvalScorerArgs[str, str]) -> float:
        return 1.0 if args.output == args.expected else 0.0

    result = Eval(
        "test_with_parameters",
        data=[
            {"input": "world", "expected": "Hi world!"},
        ],
        task=task_with_params,
        scorer=scorer,
        parameters={"prefix": "Hi", "suffix": "!"},
    )

    assert result.results[0].output == "Hi world!"
    assert result.results[0].scores["Grader"] == 1.0

    payloads = mock_scrapi.get_eval_payloads()
    assert len(payloads) == 1
    assert payloads[0]["results"][0]["output"] == "Hi world!"
    assert payloads[0]["results"][0]["scores"]["Grader"] == 1.0


def test_eval_missing_api_key(httpserver: HTTPServer, monkeypatch):
    monkeypatch.delenv("STATSIG_API_KEY", raising=False)

    def scorer(args: EvalScorerArgs[str, str]) -> float:
        return 1.0

    with pytest.raises(RuntimeError, match="Missing STATSIG_API_KEY"):
        Eval(
            "test_missing_api_key",
            data=[{"input": "world"}],
            task=lambda input: f"Hello {input}",
            scorer=scorer,
        )


def test_eval_with_summary_scorer(eval_setup):
    mock_scrapi = eval_setup

    def scorer(args: EvalScorerArgs[str, str]) -> float:
        return 1.0 if args.output == args.expected else 0.0

    def summary_scorer(results):
        total_score = 0.0
        count = 0
        for result in results:
            for score_value in result.scores.values():
                total_score += score_value
                count += 1
        avg_score = total_score / count if count > 0 else 0.0

        pass_count = sum(1 for r in results if all(s == 1.0 for s in r.scores.values()))
        pass_rate = pass_count / len(results) if results else 0.0

        return {
            "average_score": avg_score,
            "pass_rate": pass_rate,
            "total_count": len(results),
        }

    result = Eval(
        "test_with_summary_scorer",
        data=[
            {"input": "world", "expected": "Hello world"},
            {"input": "test", "expected": "Hello test"},
            {"input": "foo", "expected": "Goodbye foo"},
        ],
        task=lambda input: f"Hello {input}",
        scorer=scorer,
        summary_score_fn=summary_scorer,
    )

    assert result.summary_scores is not None
    assert result.summary_scores["average_score"] == pytest.approx(2.0 / 3.0)
    assert result.summary_scores["pass_rate"] == pytest.approx(2.0 / 3.0)
    assert result.summary_scores["total_count"] == 3

    payloads = mock_scrapi.get_eval_payloads()
    assert len(payloads) == 1
    assert "summaryScores" in payloads[0]
    assert payloads[0]["summaryScores"]["average_score"] == pytest.approx(2.0 / 3.0)
    assert payloads[0]["summaryScores"]["pass_rate"] == pytest.approx(2.0 / 3.0)
    assert payloads[0]["summaryScores"]["total_count"] == 3


def test_eval_without_summary_scorer(eval_setup):
    mock_scrapi = eval_setup

    def scorer(args: EvalScorerArgs[str, str]) -> float:
        return 1.0

    result = Eval(
        "test_without_summary_scorer",
        data=[
            {"input": "world", "expected": "Hello world"},
        ],
        task=lambda input: f"Hello {input}",
        scorer=scorer,
    )

    assert result.summary_scores is None
    assert result.results[0].input == "world"
    assert result.results[0].output == "Hello world"
    assert result.results[0].scores["Grader"] == 1.0

    payloads = mock_scrapi.get_eval_payloads()
    assert len(payloads) == 1
    assert "summaryScores" not in payloads[0]
    assert payloads[0]["results"][0]["input"] == "world"
    assert payloads[0]["results"][0]["output"] == "Hello world"
    assert payloads[0]["results"][0]["scores"]["Grader"] == 1.0


def test_eval_summary_scorer_with_multiple_scorers(eval_setup):
    mock_scrapi = eval_setup

    def exact_match_scorer(args: EvalScorerArgs[str, str]) -> bool:
        return args.output == args.expected

    def length_scorer(args: EvalScorerArgs[str, str]) -> int:
        return len(args.output) > 5

    def summary_scorer(results):
        exact_match_scores = []
        length_scores = []

        for result in results:
            exact_match_scores.append(result.scores.get("exact_match", 0.0))
            length_scores.append(result.scores.get("length", 0.0))

        return {
            "avg_exact_match": sum(exact_match_scores) / len(exact_match_scores),
            "avg_length": sum(length_scores) / len(length_scores),
            "min_exact_match": min(exact_match_scores),
            "max_length": max(length_scores),
        }

    result = Eval(
        "test_summary_scorer_multiple_scorers",
        data=[
            {"input": "world", "expected": "Hello world"},
            {"input": "test", "expected": "Hello test"},
            {"input": "a", "expected": "Hello b"},
        ],
        task=lambda input: f"Hello {input}",
        scorer={
            "exact_match": exact_match_scorer,
            "length": length_scorer,
        },
        summary_score_fn=summary_scorer,
    )

    assert result.results[0].input == "world"
    assert result.results[0].output == "Hello world"
    assert result.results[0].scores["exact_match"] == 1.0
    assert result.results[0].scores["length"] == 1.0
    assert result.results[2].input == "a"
    assert result.results[2].output == "Hello a"
    assert result.results[2].scores["exact_match"] == 0.0
    assert result.results[2].scores["length"] == 1.0

    assert result.summary_scores is not None
    assert result.summary_scores["avg_exact_match"] == pytest.approx(2.0 / 3.0)
    assert result.summary_scores["avg_length"] == 1.0
    assert result.summary_scores["min_exact_match"] == 0.0
    assert result.summary_scores["max_length"] == 1.0

    payloads = mock_scrapi.get_eval_payloads()
    assert len(payloads) == 1
    assert "summaryScores" in payloads[0]
    assert payloads[0]["results"][0]["input"] == "world"
    assert payloads[0]["results"][0]["output"] == "Hello world"
    assert payloads[0]["results"][0]["scores"]["exact_match"] == 1.0
    assert payloads[0]["results"][0]["scores"]["length"] == 1.0
    assert payloads[0]["summaryScores"]["avg_exact_match"] == pytest.approx(2.0 / 3.0)
    assert payloads[0]["summaryScores"]["avg_length"] == 1.0
    assert payloads[0]["summaryScores"]["min_exact_match"] == 0.0
    assert payloads[0]["summaryScores"]["max_length"] == 1.0


def test_eval_summary_scorer_error_handling(eval_setup):
    mock_scrapi = eval_setup

    def scorer(args: EvalScorerArgs[str, str]) -> float:
        return 1.0

    def failing_summary_scorer(results):
        raise ValueError("Summary scorer failed")

    result = Eval(
        "test_summary_scorer_error",
        data=[
            {"input": "world", "expected": "Hello world"},
        ],
        task=lambda input: f"Hello {input}",
        scorer=scorer,
        summary_score_fn=failing_summary_scorer,
    )

    assert result.summary_scores is None
    assert result.results[0].input == "world"
    assert result.results[0].output == "Hello world"
    assert result.results[0].scores["Grader"] == 1.0

    payloads = mock_scrapi.get_eval_payloads()
    assert len(payloads) == 1
    assert "results" in payloads[0]
    assert len(payloads[0]["results"]) == 1
    assert payloads[0]["results"][0]["input"] == "world"
    assert payloads[0]["results"][0]["output"] == "Hello world"
    assert payloads[0]["results"][0]["scores"]["Grader"] == 1.0
    assert "summaryScores" not in payloads[0]


def test_eval_summary_scorer_with_categories(eval_setup):
    mock_scrapi = eval_setup

    def scorer(args: EvalScorerArgs[str, str]) -> float:
        return 1.0 if args.output == args.expected else 0.0

    def summary_scorer(results):
        category_scores = {}
        for result in results:
            category = result.category if result.category else "uncategorized"
            if isinstance(category, list):
                category = "_".join(category)

            if category not in category_scores:
                category_scores[category] = []

            category_scores[category].append(
                sum(result.scores.values()) / len(result.scores) if result.scores else 0.0
            )

        summary = {}
        for category, scores in category_scores.items():
            summary[f"{category}_avg"] = sum(scores) / len(scores)

        return summary

    result = Eval(
        "test_summary_scorer_categories",
        data=[
            {"input": "world", "expected": "Hello world", "category": "greeting"},
            {"input": "test", "expected": "Hello test", "category": "greeting"},
            {"input": "foo", "expected": "Goodbye foo", "category": "farewell"},
        ],
        task=lambda input: f"Hello {input}",
        scorer=scorer,
        summary_score_fn=summary_scorer,
    )

    assert result.results[0].input == "world"
    assert result.results[0].output == "Hello world"
    assert result.results[0].category == "greeting"
    assert result.results[0].scores["Grader"] == 1.0
    assert result.results[2].input == "foo"
    assert result.results[2].output == "Hello foo"
    assert result.results[2].category == "farewell"
    assert result.results[2].scores["Grader"] == 0.0

    assert result.summary_scores is not None
    assert "greeting_avg" in result.summary_scores
    assert "farewell_avg" in result.summary_scores
    assert result.summary_scores["greeting_avg"] == 1.0
    assert result.summary_scores["farewell_avg"] == 0.0

    payloads = mock_scrapi.get_eval_payloads()
    assert len(payloads) == 1
    assert "summaryScores" in payloads[0]
    assert payloads[0]["results"][0]["input"] == "world"
    assert payloads[0]["results"][0]["output"] == "Hello world"
    assert payloads[0]["results"][0]["category"] == "greeting"
    assert payloads[0]["results"][0]["scores"]["Grader"] == 1.0
    assert payloads[0]["results"][2]["input"] == "foo"
    assert payloads[0]["results"][2]["output"] == "Hello foo"
    assert payloads[0]["results"][2]["scores"]["Grader"] == 0.0
    assert "greeting_avg" in payloads[0]["summaryScores"]
    assert "farewell_avg" in payloads[0]["summaryScores"]
    assert payloads[0]["summaryScores"]["greeting_avg"] == 1.0
    assert payloads[0]["summaryScores"]["farewell_avg"] == 0.0


def test_eval_summary_scorer_with_empty_results(eval_setup):
    mock_scrapi = eval_setup

    def scorer(args: EvalScorerArgs[str, str]) -> float:
        return 1.0

    def summary_scorer(results):
        if not results:
            return {"count": 0, "message": "no_results"}
        return {"count": len(results)}

    result = Eval(
        "test_summary_scorer_empty",
        data=[],
        task=lambda input: f"Hello {input}",
        scorer=scorer,
        summary_score_fn=summary_scorer,
    )

    assert len(result.results) == 0
    assert result.summary_scores is not None
    assert result.summary_scores["count"] == 0
    assert result.summary_scores["message"] == "no_results"

    payloads = mock_scrapi.get_eval_payloads()
    assert len(payloads) == 1
    assert len(payloads[0]["results"]) == 0
    assert "summaryScores" in payloads[0]
    assert payloads[0]["summaryScores"]["count"] == 0
    assert payloads[0]["summaryScores"]["message"] == "no_results"


def test_eval_summary_scorer_with_task_errors(eval_setup):
    mock_scrapi = eval_setup

    def failing_task(input: str) -> str:
        if input == "fail":
            raise ValueError("Task failed")
        return f"Hello {input}"

    def scorer(args: EvalScorerArgs[str, str]) -> float:
        return 1.0

    def summary_scorer(results):
        error_count = sum(1 for r in results if r.error)
        success_count = len(results) - error_count

        total_score = sum(
            sum(r.scores.values()) / len(r.scores) if r.scores else 0.0 for r in results
        )
        avg_score = total_score / len(results) if results else 0.0

        success_scores = [
            sum(r.scores.values()) / len(r.scores) for r in results if not r.error and r.scores
        ]
        avg_success_score = sum(success_scores) / len(success_scores) if success_scores else 0.0

        return {
            "error_count": error_count,
            "success_count": success_count,
            "avg_score": avg_score,
            "avg_success_score": avg_success_score,
        }

    result = Eval(
        "test_summary_scorer_task_errors",
        data=[
            {"input": "world", "expected": "Hello world"},
            {"input": "fail", "expected": "Hello fail"},
            {"input": "test", "expected": "Hello test"},
        ],
        task=failing_task,
        scorer=scorer,
        summary_score_fn=summary_scorer,
    )

    assert result.results[0].input == "world"
    assert result.results[0].output == "Hello world"
    assert result.results[0].error == False
    assert result.results[0].scores["Grader"] == 1.0
    assert result.results[1].input == "fail"
    assert result.results[1].error == True
    assert result.results[1].scores["Grader"] == 0.0
    assert result.results[2].input == "test"
    assert result.results[2].output == "Hello test"
    assert result.results[2].error == False
    assert result.results[2].scores["Grader"] == 1.0

    assert result.summary_scores is not None
    assert result.summary_scores["error_count"] == 1
    assert result.summary_scores["success_count"] == 2
    assert result.summary_scores["avg_score"] == pytest.approx(2.0 / 3.0)
    assert result.summary_scores["avg_success_score"] == 1.0

    payloads = mock_scrapi.get_eval_payloads()
    assert len(payloads) == 1
    assert "summaryScores" in payloads[0]
    assert payloads[0]["results"][0]["input"] == "world"
    assert payloads[0]["results"][0]["output"] == "Hello world"
    assert payloads[0]["results"][0]["error"] == False
    assert payloads[0]["results"][0]["scores"]["Grader"] == 1.0
    assert payloads[0]["results"][1]["input"] == "fail"
    assert payloads[0]["results"][1]["error"] == True
    assert payloads[0]["results"][1]["scores"]["Grader"] == 0.0
    assert payloads[0]["results"][2]["input"] == "test"
    assert payloads[0]["results"][2]["output"] == "Hello test"
    assert payloads[0]["results"][2]["scores"]["Grader"] == 1.0
    assert payloads[0]["summaryScores"]["error_count"] == 1
    assert payloads[0]["summaryScores"]["success_count"] == 2
    assert payloads[0]["summaryScores"]["avg_score"] == pytest.approx(2.0 / 3.0)
    assert payloads[0]["summaryScores"]["avg_success_score"] == 1.0


def test_eval_summary_scorer_throws_during_computation(eval_setup):
    mock_scrapi = eval_setup

    def scorer(args: EvalScorerArgs[str, str]) -> float:
        return 1.0 if args.output == args.expected else 0.0

    def failing_summary_scorer(results):
        total = sum(r.scores.get("Grader", 0.0) for r in results)
        raise RuntimeError("Failed during summary computation")

    result = Eval(
        "test_summary_scorer_throws",
        data=[
            {"input": "world", "expected": "Hello world"},
            {"input": "test", "expected": "Hello test"},
        ],
        task=lambda input: f"Hello {input}",
        scorer=scorer,
        summary_score_fn=failing_summary_scorer,
    )

    assert result.results[0].input == "world"
    assert result.results[0].output == "Hello world"
    assert result.results[0].scores["Grader"] == 1.0
    assert result.results[1].input == "test"
    assert result.results[1].output == "Hello test"
    assert result.results[1].scores["Grader"] == 1.0
    assert result.summary_scores is None

    payloads = mock_scrapi.get_eval_payloads()
    assert len(payloads) == 1
    assert len(payloads[0]["results"]) == 2
    assert payloads[0]["results"][0]["input"] == "world"
    assert payloads[0]["results"][0]["output"] == "Hello world"
    assert payloads[0]["results"][0]["scores"]["Grader"] == 1.0
    assert payloads[0]["results"][1]["input"] == "test"
    assert payloads[0]["results"][1]["output"] == "Hello test"
    assert payloads[0]["results"][1]["scores"]["Grader"] == 1.0
    assert "summaryScores" not in payloads[0]



from statsig_python_core import Statsig, StatsigOptions, StatsigUser
from .prompt import PromptEvaluationOptions, Prompt
from .ai_eval_grade_data import AIEvalGradeData
from .prompt_version import PromptVersion
from .statsig_ai import StatsigAI
from .statsig_ai_base import StatsigCreateConfig, StatsigAttachConfig
from .evals import Eval, EvalScorerArgs, EvalDataRecord, EvalHook

__all__ = [
    "StatsigAI",
    "StatsigCreateConfig",
    "StatsigAttachConfig",
    # Serving
    "Prompt",
    "PromptVersion",
    "PromptEvaluationOptions",
    # Logging
    "AIEvalGradeData",
    # Re-export from statsig-python-core for convenience
    "Statsig",
    "StatsigOptions",
    "StatsigUser",
    # Evals
    "Eval",
    "EvalScorerArgs",
    "EvalDataRecord",
    "EvalHook",
]



import asyncio
import inspect
import logging
import os
import urllib.parse
from typing import (
    Any,
    AsyncIterator,
    Awaitable,
    Callable,
    Dict,
    Iterable,
    List,
    Optional,
    TypeVar,
    Union,
    cast,
)

import requests

from .eval_types import (
    EvalDataRecord,
    EvalHook,
    EvalResult,
    EvalResultMetadata,
    EvalScorerArgs,
    Input,
    ScorerFnMap,
    Scorer,
    Output,
    Score,
    EvalData,
    EvalTask,
    EvalScorer,
    EvalParameters,
    _DataclassOrDictEvalDataRecord,
    EvalResultRecord,
    SummaryScorerFn,
)


T = TypeVar("T")


async def to_async_iter(iterable: Iterable[T]) -> AsyncIterator[T]:
    for item in iterable:
        yield item


def _normalize_record(
    record: _DataclassOrDictEvalDataRecord,
) -> EvalDataRecord[Input, Output]:
    """Convert a dict or dataclass record into EvalDataRecord."""
    if isinstance(record, EvalDataRecord):
        return record

    if isinstance(record, dict):
        return EvalDataRecord(
            input=cast(Input, record.get("input")),
            category=record.get("category"),
            expected=cast(Optional[Output], record.get("expected")),
        )

    raise TypeError(f"Invalid data record type: {type(record)}")


async def _normalize_into_data_iterator(
    data: EvalData,
) -> AsyncIterator[EvalDataRecord[Input, Output]]:
    data_iterator: Any = data
    if inspect.isclass(data_iterator):
        data_iterator = data_iterator()

    if inspect.isfunction(data_iterator) or inspect.isroutine(data_iterator):
        data_iterator = data_iterator()

    if not inspect.isasyncgen(data_iterator):
        data_iterator = to_async_iter(data_iterator)

    async for record in data_iterator:
        yield _normalize_record(record)


def _normalize_scorers(scorers: EvalScorer) -> ScorerFnMap[Input, Output]:
    scorers_dict = scorers if isinstance(scorers, dict) else {"Grader": scorers}
    for name, scorer in scorers_dict.items():
        if inspect.isclass(scorer):
            scorers_dict[name] = scorer()
    return scorers_dict


def _normalize_score_value(score_value: Score) -> float:
    """Convert a score to a consistent float value."""
    if isinstance(score_value, bool):
        return 1.0 if score_value else 0.0
    if isinstance(score_value, (int, float)):
        return float(score_value)
    logging.warning("[Statsig] Invalid score type: %s", type(score_value))
    try:
        return float(score_value)
    except (ValueError, TypeError):
        return 0.0


def _send_eval_results(
    name: str,
    records: List[EvalResultRecord[Input, Output]],
    api_key: str,
    eval_run_name: Optional[str],
    summary_scores: Optional[Dict[str, float]],
) -> None:
    """Send eval results to Statsig API (synchronous, using requests)."""

    url = f"https://api.statsig.com/console/v1/evals/send_results/{urllib.parse.quote(name)}"
    record_as_dicts = []
    try:
        record_as_dicts = [record.to_dict() for record in records]
    except Exception as err:
        logging.warning("[Statsig] Failed to convert records to dicts: %s", err)
        record_as_dicts = []

    payload: Dict[str, Any] = {"results": record_as_dicts}
    if eval_run_name:
        payload["name"] = eval_run_name
    if summary_scores:
        payload["summaryScores"] = summary_scores
    headers = {
        "Content-Type": "application/json",
        "STATSIG-API-KEY": api_key,
    }

    try:
        response = requests.post(url, json=payload, headers=headers, timeout=10)
        if response.ok:
            logging.info(
                "[Statsig] Sent eval results (%d records): %s",
                len(records),
                response.reason,
            )
        else:
            logging.warning(
                "[Statsig] Failed to send eval results: %s %s - %s",
                response.status_code,
                response.reason,
                response.text,
            )
    except requests.RequestException as error:
        logging.error("[Statsig] Error sending eval results: %s", error)


def _run_summary_scorer(
    summary_scorer: Optional[SummaryScorerFn[Input, Output]],
    results: List[EvalResultRecord[Input, Output]],
) -> Optional[Dict[str, float]]:
    try:
        if summary_scorer:
            return summary_scorer(results)
        return None
    except Exception as err:
        logging.warning("[Statsig] Summary scorer failed: %s", err)
        return None


async def _run_task(
    task: EvalTask[Input, Output],
    input_value: Input,
    hook: EvalHook,
) -> Output:
    sig = inspect.signature(task)
    params = list(sig.parameters.values())
    num_args = len(params)

    if num_args == 1:
        args: Any = (input_value,)
    elif num_args >= 2:
        args = (input_value, hook)
    else:
        raise TypeError(f"Invalid task signature: expected 1 or 2 parameters, got {num_args}")

    if inspect.iscoroutinefunction(task):
        return await task(*args)

    event_loop = asyncio.get_event_loop()

    def sync_call() -> Output:
        return task(*args)  # type: ignore

    return await event_loop.run_in_executor(None, sync_call)


async def _run_scorer(
    scorer_name: str,
    scorer: Scorer[Input, Output],
    output: Output,
    expected: Optional[Output],
    input_value: Input,
) -> float:
    """Run a single scorer and return normalized score as float."""
    try:
        args = EvalScorerArgs(input=input_value, output=output, expected=expected)

        # Check for AsyncScorer protocol (has a score method)
        if hasattr(scorer, "score") and callable(getattr(scorer, "score", None)):
            score_method = getattr(scorer, "score")
            raw_score = score_method(args)
        elif callable(scorer):
            # SyncScorer or callable function
            scorer_fn = cast(
                Callable[[EvalScorerArgs[Input, Output]], Union[Score, Awaitable[Score]]], scorer
            )
            raw_score = scorer_fn(args)
        else:
            raise TypeError(f"Invalid scorer type: {type(scorer)}")

        if inspect.isawaitable(raw_score):
            raw_score = await raw_score

        return _normalize_score_value(raw_score)
    except Exception as err:
        logging.warning(
            "[Statsig] Scorer '%s' failed: %s | input=%r",
            scorer_name,
            err,
            input_value,
        )
        return 0.0


async def _run_task_and_score_one_record(
    task: EvalTask[Input, Output],
    scorers: ScorerFnMap[Input, Output],
    data_record: EvalDataRecord[Input, Output],
    parameters: Optional[EvalParameters],
) -> EvalResultRecord[Input, Optional[Output]]:
    """Run task and score for one record asynchronously.

    Runs the task first, then runs all scorers concurrently on the output.
    - If task errors: all scores are "error" and error flag is set
    - If a scorer errors: that score is "0"
    """
    eval_result: EvalResultRecord[Input, Optional[Output]] = EvalResultRecord(
        input=data_record.input,
        expected=data_record.expected,
        category=data_record.category,
        output=None,
        scores={},
        error=False,
    )

    try:
        hook = EvalHook(parameters=parameters, category=data_record.category)
        output = await _run_task(task, data_record.input, hook)
        eval_result.output = output

        scorer_tasks = [
            _run_scorer(scorer_name, scorer_fn, output, data_record.expected, data_record.input)
            for scorer_name, scorer_fn in scorers.items()
        ]

        score_results = await asyncio.gather(*scorer_tasks)

        eval_result.scores = dict(zip(scorers.keys(), score_results))

    except Exception as err:
        # Task failed - all scores should be nan, scoring errors are caught in _run_scorer
        logging.warning("[Statsig] Task failed: %s | input=%r", err, data_record.input)
        eval_result.error = True
        eval_result.scores = {name: 0.0 for name in scorers.keys()}

    return eval_result


def Eval(
    name: str,
    *,
    data: EvalData[Input, Output],
    task: Union[EvalTask[Input, Output]],
    scorer: EvalScorer[Input, Output],
    parameters: Optional[EvalParameters] = None,
    eval_run_name: Optional[str] = None,
    summary_score_fn: Optional[SummaryScorerFn[Input, Output]] = None,
) -> EvalResult[Input, Output]:
    """
    Run evaluation synchronously with support for async tasks, scorers, and data.

    Args:
        name: Name of the evaluation
        data: Evaluation data (iterable of records, can be async)
        task: Task function(s) to run on each input (can be async, can be list)
        scorer: Scorer function(s) to evaluate outputs (can be async, can be dict)
        parameters: Optional parameters to pass to tasks
        eval_run_name: Optional name for this eval run
        summary_score_fn: Optional function that takes a list of results and returns a dictionary of summary scores

    Returns:
        Dictionary with 'results' and 'metadata' keys
    """
    api_key = os.environ.get("STATSIG_API_KEY")
    if not api_key:
        raise RuntimeError("[Statsig] Missing STATSIG_API_KEY environment variable")

    data_iterator: AsyncIterator[EvalDataRecord[Input, Output]] = _normalize_into_data_iterator(
        data
    )
    normalized_scorers: ScorerFnMap[Input, Output] = _normalize_scorers(scorer)

    async def run_all():
        tasks_to_run = []
        async for record in data_iterator:
            tasks_to_run.append(
                _run_task_and_score_one_record(task, normalized_scorers, record, parameters)
            )
        return await asyncio.gather(*tasks_to_run)

    results = asyncio.run(run_all())

    any_error = any(r.error for r in results)

    summary_scores = _run_summary_scorer(summary_score_fn, results)

    _send_eval_results(name, results, api_key, eval_run_name, summary_scores)

    return EvalResult(
        results=results,
        metadata=EvalResultMetadata(error=any_error),
        summary_scores=summary_scores,
    )



import os
from typing import Optional, Dict
from dataclasses import dataclass

from opentelemetry import trace, context
from opentelemetry.context.context import Context
from opentelemetry.sdk.trace import TracerProvider
from opentelemetry.sdk.trace.sampling import ALWAYS_ON
from opentelemetry.sdk.resources import Resource

from .exporter import StatsigOTLPTraceExporter, StatsigOTLPTraceExporterOptions
from .processor import StatsigSpanProcessor
from .singleton import OtelSingleton


class InitializeTracingOptions:
    def __init__(
        self,
        global_context_manager: Optional[Context] = None,
        skip_global_context_manager_setup: bool = False,
        enable_global_trace_provider_registration: bool = False,
        global_trace_provider: Optional[TracerProvider] = None,
        exporter_options: Optional[StatsigOTLPTraceExporterOptions] = None,
        service_name: Optional[str] = None,
        version: Optional[str] = None,
        environment: Optional[str] = None,
    ):
        self.global_context_manager = global_context_manager
        self.skip_global_context_manager_setup = skip_global_context_manager_setup
        self.enable_global_trace_provider_registration = enable_global_trace_provider_registration
        self.global_trace_provider = global_trace_provider
        self.exporter_options = exporter_options or StatsigOTLPTraceExporterOptions()
        self.service_name = service_name
        self.version = version
        self.environment = environment


@dataclass
class InitializeTracingResult:
    exporter: StatsigOTLPTraceExporter
    processor: StatsigSpanProcessor
    provider: TracerProvider


def initialize_tracing(
    options: Optional[InitializeTracingOptions] = None,
) -> InitializeTracingResult:
    options = options or InitializeTracingOptions()

    if not options.global_context_manager and not options.skip_global_context_manager_setup:
        try:
            context.attach(context.get_current())
        except Exception:
            print(
                "Could not automatically set up a global OTEL context manager.\n"
                "This may be expected if you or another library already set one.\n"
                "You can skip this setup by passing skip_global_context_manager_setup=True."
            )

    trace_components = _create_trace_components(
        exporter_options=options.exporter_options,
        resources={
            "service.name": options.service_name or os.getenv("OTEL_SERVICE_NAME"),
            "service.version": options.version,
            "env": options.environment or os.getenv("PYTHON_ENV"),
        },
    )

    tracer_provider = options.global_trace_provider or trace_components.provider

    OtelSingleton.instantiate(tracer_provider)

    if options.enable_global_trace_provider_registration:
        trace.set_tracer_provider(tracer_provider)

    return trace_components


def _create_trace_components(
    exporter_options: StatsigOTLPTraceExporterOptions,
    resources: Dict[str, Optional[str]],
) -> InitializeTracingResult:
    exporter = StatsigOTLPTraceExporter(
        options=exporter_options,
    )

    processor = StatsigSpanProcessor(exporter)

    resource = Resource.create({k: v for k, v in resources.items() if v is not None})

    provider = TracerProvider(
        resource=resource,
        sampler=ALWAYS_ON,
    )
    provider.add_span_processor(processor)

    return InitializeTracingResult(
        exporter=exporter,
        processor=processor,
        provider=provider,
    )



from typing import Any, Dict, Optional
import json
import logging
from opentelemetry.trace import Span, Status, StatusCode
from statsig_python_core import Statsig, StatsigUser

GEN_AI_EVENT_NAME = "statsig::gen_ai"
PLACEHOLDER_STATSIG_USER = StatsigUser(user_id="statsig-ai-openai-wrapper")

logger = logging.getLogger(__name__)


class SpanTelemetry:
    def __init__(
        self,
        span: Span,
        span_name: str,
        max_json_chars: int,
        provider_name: str,
    ):
        self.span = span
        self.span_name = span_name
        self.max_json_chars = max_json_chars
        self.metadata: Dict[str, str] = {}
        self.ended = False

        self.metadata["gen_ai.provider.name"] = provider_name
        ctx = span.get_span_context()
        self.metadata["span.trace_id"] = str(ctx.trace_id)
        self.metadata["span.span_id"] = str(ctx.span_id)

    def set_operation_name(self, operation_name: str) -> None:
        self.metadata["gen_ai.operation.name"] = operation_name

    def set_attributes(self, kv: Dict[str, Any]) -> None:
        for key, value in kv.items():
            if value is None:
                continue
            if isinstance(value, (list, dict)):
                self._set_json_safe(key, value)
                continue

            self.span.set_attribute(key, value)
            self.metadata[key] = attribute_value_to_metadata(value)

    def _set_json_safe(self, key: str, value: Any) -> None:
        try:
            json_str = json.dumps(value if value is not None else None)
            if len(json_str) > self.max_json_chars:
                truncated = json_str[: self.max_json_chars] + "…(truncated)"
                # Directly set attributes instead of calling set_attributes again
                self.span.set_attribute(key, truncated)
                self.span.set_attribute(f"{key}_truncated", True)
                self.span.set_attribute(f"{key}_len", len(json_str))

                self.metadata[key] = truncated
                self.metadata[f"{key}_truncated"] = "True"
                self.metadata[f"{key}_len"] = str(len(json_str))
            else:
                self.span.set_attribute(key, json_str)
                self.metadata[key] = json_str
        except Exception:
            self.span.set_attribute(key, "[[unserializable]]")
            self.metadata[key] = "[[unserializable]]"

    def set_status(self, status: Dict[str, Any]) -> None:
        code: StatusCode = status.get("code", StatusCode.UNSET)
        message: Optional[str] = status.get("message")

        self.span.set_status(Status(code, message))
        if code == StatusCode.ERROR:
            self.metadata["error.type"] = type(message).__name__ if message else "Error"
        self.metadata["span.status_code"] = code.name
        self.metadata["span.status_code_value"] = str(code.value)
        if message:
            self.metadata["span.status_message"] = str(message)

    def record_exception(self, error: Any) -> None:
        self.span.record_exception(error)
        type_name = getattr(error, "__class__", None)
        if type_name:
            self.metadata["exception.type"] = type_name.__name__
        if hasattr(error, "message"):
            self.metadata["exception.message"] = str(error.message)
        elif isinstance(error, str):
            self.metadata["exception.message"] = error

    def fail(self, error: Any) -> None:
        self.record_exception(error)
        msg = getattr(error, "message", str(error))
        self.set_status({"code": StatusCode.ERROR, "message": msg})

    def end(self) -> None:
        if self.ended:
            return
        self.ended = True
        self.span.end()
        self._log_span_event(self.span_name, dict(self.metadata))

    def _log_span_event(self, span_name: str, metadata: Dict[str, str]) -> None:
        statsig = get_statsig_instance_for_logging()
        if not statsig:
            logger.warning(
                "[Statsig] No shared global StatsigAI instance found. "
                "Call StatsigAI.new_shared() before invoking OpenAI methods to capture Gen AI telemetry."
            )
            return

        try:
            statsig.log_event(
                PLACEHOLDER_STATSIG_USER,
                GEN_AI_EVENT_NAME,
                span_name,
                metadata,
            )
        except Exception as e:
            logger.warning("[Statsig] Failed to log span event: %s", str(e))


def attribute_value_to_metadata(value: Any) -> str:
    if value is None:
        return "null"
    if isinstance(value, (list, dict)):
        return safe_stringify(value)
    if isinstance(value, (str, int, float, bool)):
        return str(value)
    return str(value)


def safe_stringify(value: Any) -> str:
    try:
        return json.dumps(value)
    except Exception:
        return str(value)


def get_statsig_instance_for_logging() -> Optional[Statsig]:
    try:
        from ..statsig_ai import StatsigAI  # pylint: disable=import-outside-toplevel

        if hasattr(StatsigAI, "has_shared_instance") and StatsigAI.has_shared_instance():
            statsig_instance = StatsigAI.shared()
            return statsig_instance.get_statsig()
    except Exception as e:
        logger.warning("[Statsig] Unable to retrieve Statsig instance for span logging: %s", str(e))
    return None



import pytest
import json
from pytest_httpserver import HTTPServer
from mock_scrapi import MockScrapi
from utils import get_test_data_resource
from statsig_ai import (
    StatsigOptions,
    StatsigAI,
    StatsigCreateConfig,
)
from statsig_ai.wrappers import wrap_openai, WrapOpenAIOptions, GenAICaptureOptions
from mock_openai import MockOpenAI


@pytest.fixture
def statsig_setup(httpserver: HTTPServer):
    mock_scrapi = MockScrapi(httpserver)
    dcs_content = get_test_data_resource("eval_proj_dcs.json")

    mock_scrapi.stub(
        "/v2/download_config_specs/secret-key.json", response=dcs_content, method="GET"
    )
    mock_scrapi.stub("/v1/log_event", response='{"success": true}', method="POST")

    options = StatsigOptions(
        specs_url=mock_scrapi.url_for_endpoint("/v2/download_config_specs"),
        log_event_url=mock_scrapi.url_for_endpoint("/v1/log_event"),
    )

    yield options, mock_scrapi


def test_wrap_openai(statsig_setup):
    oai_client = MockOpenAI()
    wrapped_client = wrap_openai(oai_client)
    assert wrapped_client is not None


def test_wrap_openai_chat_completions(statsig_setup):
    options, mock_scrapi = statsig_setup
    StatsigAI.new_shared(
        statsig_source=StatsigCreateConfig(sdk_key="secret-key", statsig_options=options)
    )
    StatsigAI.shared().initialize()
    oai_client = MockOpenAI()
    wrapped_client = wrap_openai(oai_client)
    response = wrapped_client.chat.completions.create(
        model="gpt-4",
        messages=[{"role": "user", "content": "Hello, world!"}],
        temperature=0.7,
        max_tokens=100,
    )
    StatsigAI.shared().shutdown()
    assert response["choices"][0]["message"]["content"] == "This is a mock chat response"
    gen_ai_events = [
        e for e in mock_scrapi.get_logged_events() if e["eventName"] == "statsig::gen_ai"
    ]
    assert len(gen_ai_events) == 1
    assert gen_ai_events[0]["eventName"] == "statsig::gen_ai"
    metadata = gen_ai_events[0]["metadata"]
    print(metadata)
    assert metadata["gen_ai.provider.name"] == "openai"
    assert metadata["gen_ai.operation.name"] == "chat"
    assert metadata["gen_ai.request.model"] == "gpt-4"
    assert metadata["gen_ai.response.model"] == "gpt-4"
    assert metadata["gen_ai.response.finish_reasons"] == json.dumps(["stop"])
    assert metadata["gen_ai.usage.input_tokens"] == "5"
    assert metadata["gen_ai.usage.output_tokens"] == "7"


def test_respects_capture_options(statsig_setup):
    options, mock_scrapi = statsig_setup
    StatsigAI.new_shared(
        statsig_source=StatsigCreateConfig(sdk_key="secret-key", statsig_options=options)
    )
    StatsigAI.shared().initialize()
    oai_client = MockOpenAI()
    wrapped_client = wrap_openai(
        oai_client,
        options=WrapOpenAIOptions(
            gen_ai_capture_options=GenAICaptureOptions(capture_input_messages=True)
        ),
    )
    response = wrapped_client.chat.completions.create(
        model="gpt-4",
        messages=[{"role": "user", "content": "Hello, world!"}],
        temperature=0.7,
        max_tokens=100,
    )
    StatsigAI.shared().shutdown()
    assert response["choices"][0]["message"]["content"] == "This is a mock chat response"
    gen_ai_events = [
        e for e in mock_scrapi.get_logged_events() if e["eventName"] == "statsig::gen_ai"
    ]
    assert len(gen_ai_events) == 1
    assert gen_ai_events[0]["eventName"] == "statsig::gen_ai"
    metadata = gen_ai_events[0]["metadata"]
    assert metadata["gen_ai.input.messages"] == json.dumps(
        [{"role": "user", "content": "Hello, world!"}]
    )
    assert metadata.get("gen_ai.output.messages") is None



import os
from typing import Optional

from opentelemetry.exporter.otlp.proto.http.trace_exporter import OTLPSpanExporter


class StatsigOTLPTraceExporterOptions:
    def __init__(self, sdk_key: Optional[str] = None, dsn: Optional[str] = None):
        self.sdk_key = sdk_key
        self.dsn = dsn


class StatsigOTLPTraceExporter(OTLPSpanExporter):
    def __init__(self, options: StatsigOTLPTraceExporterOptions):
        sdk_key = options.sdk_key or os.getenv("STATSIG_SDK_KEY")
        if not sdk_key:
            raise ValueError("Statsig SDK Key is required for StatsigOTLPTraceExporter")

        dsn = options.dsn or "https://api.statsig.com/otlp"

        super().__init__(
            endpoint=f"{dsn.rstrip('/')}/v1/traces",
            headers={"statsig-api-key": sdk_key},
        )



import logging
from typing import Any
from opentelemetry import trace
from opentelemetry.trace import SpanKind, StatusCode
from .configs import WrapOpenAIOptions, GenAICaptureOptions
from .span_telemetry import SpanTelemetry
from .genai_attribute_helper import extract_genai_attributes, extract_opt_in_attributes

logger = logging.getLogger(__name__)
tracer = trace.get_tracer("statsig-openai-proxy")


class BaseWrapper:
    def __init__(self, client: Any):
        self._client = client

    def __getattr__(self, name):
        return getattr(self._client, name)


class OpenAIWrapper(BaseWrapper):
    def __init__(self, oai_client: Any, options: WrapOpenAIOptions):
        super().__init__(oai_client)
        self._options = options.gen_ai_capture_options
        # Chat Completions
        if hasattr(oai_client, "chat") and hasattr(oai_client.chat, "completions"):
            self.chat = type(
                "ChatNamespace",
                (),
                {"completions": ChatCompletionsWrapper(oai_client.chat.completions, options)},
            )()

        # Legacy Completions
        if hasattr(oai_client, "completions"):
            self.completions = CompletionsWrapper(oai_client.completions, options)

        # Embeddings
        if hasattr(oai_client, "embeddings"):
            self.embeddings = EmbeddingsWrapper(oai_client.embeddings, options)

        # Images
        if hasattr(oai_client, "images"):
            self.images = ImagesWrapper(oai_client.images, options)

        # Responses
        if hasattr(oai_client, "responses"):
            self.responses = ResponsesWrapper(oai_client.responses, options)


class ChatCompletionsWrapper(BaseWrapper):
    operation_name = "chat"
    _options: GenAICaptureOptions

    def __init__(self, client: Any, options: WrapOpenAIOptions):
        super().__init__(client)
        self._options = options.gen_ai_capture_options

    def create(self, *args, **kwargs):
        model = kwargs.get("model", "unknown")
        span_name = f"{self.operation_name} {model}"
        span = tracer.start_span(span_name, kind=SpanKind.CLIENT)
        telemetry = SpanTelemetry(span, span_name, 40_000, provider_name="openai")
        try:
            res = self._client.create(*args, **kwargs)
            attributes = extract_genai_attributes("openai", self.operation_name, model, kwargs, res)
            opt_in_attributes = extract_opt_in_attributes(self._options, kwargs, res)
            telemetry.set_attributes(attributes)
            telemetry.set_attributes(opt_in_attributes)
            telemetry.set_status({"code": StatusCode.OK})
            return res
        except Exception as e:
            telemetry.fail(e)
            raise
        finally:
            telemetry.end()


class CompletionsWrapper(BaseWrapper):
    operation_name = "text_completion"
    _options: GenAICaptureOptions

    def __init__(self, client: Any, options: WrapOpenAIOptions):
        super().__init__(client)
        self._options = options.gen_ai_capture_options

    def create(self, *args, **kwargs):
        model = kwargs.get("model")
        span_name = f"{self.operation_name} {model}"
        span = tracer.start_span(span_name, kind=SpanKind.CLIENT)
        telemetry = SpanTelemetry(span, span_name, 40_000, provider_name="openai")
        try:
            res = self._client.create(*args, **kwargs)
            attributes = extract_genai_attributes("openai", self.operation_name, model, kwargs, res)
            opt_in_attributes = extract_opt_in_attributes(self._options, kwargs, res)
            telemetry.set_attributes(opt_in_attributes)
            telemetry.set_attributes(attributes)
            telemetry.set_status({"code": StatusCode.OK})
            return res
        except Exception as e:
            telemetry.fail(e)
            raise
        finally:
            telemetry.end()


class EmbeddingsWrapper(BaseWrapper):
    operation_name = "embeddings"
    _options: GenAICaptureOptions

    def __init__(self, client: Any, options: WrapOpenAIOptions):
        super().__init__(client)
        self._options = options.gen_ai_capture_options

    def create(self, *args, **kwargs):
        model = kwargs.get("model")
        span_name = f"{self.operation_name} {model}"
        span = tracer.start_span(span_name, kind=SpanKind.CLIENT)
        telemetry = SpanTelemetry(span, span_name, 40_000, provider_name="openai")
        try:
            res = self._client.create(*args, **kwargs)
            attributes = extract_genai_attributes("openai", self.operation_name, model, kwargs, res)
            opt_in_attributes = extract_opt_in_attributes(self._options, kwargs, res)
            telemetry.set_attributes(attributes)
            telemetry.set_attributes(opt_in_attributes)
            telemetry.set_status({"code": StatusCode.OK})
            return res
        except Exception as e:
            telemetry.fail(e)
            raise
        finally:
            telemetry.end()


class ImagesWrapper(BaseWrapper):
    operation_name = "images.generate"
    _options: GenAICaptureOptions

    def __init__(self, client: Any, options: WrapOpenAIOptions):
        super().__init__(client)
        self._options = options.gen_ai_capture_options

    def generate(self, *args, **kwargs):
        model = kwargs.get("model")
        span_name = f"{self.operation_name} {model}"
        span = tracer.start_span(span_name, kind=SpanKind.CLIENT)
        telemetry = SpanTelemetry(span, span_name, 40_000, provider_name="openai")
        try:
            res = self._client.generate(*args, **kwargs)
            attributes = extract_genai_attributes("openai", self.operation_name, model, kwargs, res)
            opt_in_attributes = extract_opt_in_attributes(self._options, kwargs, res)
            telemetry.set_attributes(attributes)
            telemetry.set_attributes(opt_in_attributes)
            telemetry.set_status({"code": StatusCode.OK})
            return res
        except Exception as e:
            telemetry.fail(e)
            raise
        finally:
            telemetry.end()


class ResponsesWrapper(BaseWrapper):
    operation_name = "responses.create"
    _options: GenAICaptureOptions

    def __init__(self, client: Any, options: WrapOpenAIOptions):
        super().__init__(client)
        self._options = options.gen_ai_capture_options

    def create(self, *args, **kwargs):
        model = kwargs.get("model")
        span_name = f"{self.operation_name} {model}"
        span = tracer.start_span(span_name, kind=SpanKind.CLIENT)
        telemetry = SpanTelemetry(span, span_name, 40_000, provider_name="openai")
        try:
            res = self._client.create(*args, **kwargs)
            attributes = extract_genai_attributes("openai", self.operation_name, model, kwargs, res)
            opt_in_attributes = extract_opt_in_attributes(self._options, kwargs, res)
            telemetry.set_attributes(attributes)
            telemetry.set_attributes(opt_in_attributes)
            telemetry.set_status({"code": StatusCode.OK})
            return res
        except Exception as e:
            telemetry.fail(e)
            raise
        finally:
            telemetry.end()



from typing import Tuple
import pytest
from mock_scrapi import MockScrapi
from statsig_ai.statsig_ai_base import StatsigAttachConfig, StatsigCreateConfig
from utils import get_test_data_resource
from pytest_httpserver import HTTPServer
from statsig_ai import StatsigAI
from statsig_python_core import Statsig, StatsigOptions, StatsigUser


@pytest.fixture
def statsig_setup(httpserver: HTTPServer):
    mock_scrapi = MockScrapi(httpserver)
    dcs_content = get_test_data_resource("eval_proj_dcs.json")
    mock_scrapi.stub(
        "/v2/download_config_specs/secret-key.json", response=dcs_content, method="GET"
    )
    mock_scrapi.stub("/v1/log_event", response='{"success": true}', method="POST")

    statsig_options = StatsigOptions()
    statsig_options.specs_url = mock_scrapi.url_for_endpoint("/v2/download_config_specs")
    statsig_options.log_event_url = mock_scrapi.url_for_endpoint("/v1/log_event")

    StatsigAI.remove_shared()

    yield statsig_options, mock_scrapi

    if StatsigAI.has_shared_instance():
        statsig_ai = StatsigAI.shared()
        statsig_ai.shutdown()


def test_creating_shared_instance_with_create_config(
    statsig_setup: Tuple[StatsigOptions, MockScrapi],
):
    statsig_options, _ = statsig_setup

    statsig_ai = StatsigAI.new_shared(
        statsig_source=StatsigCreateConfig(sdk_key="secret-key", statsig_options=statsig_options)
    )
    statsig_ai.initialize()
    assert statsig_ai.get_statsig().check_gate(StatsigUser("my_user"), "test_public")


def test_getting_shared_instance_with_create_config(statsig_setup):
    statsig_options, _ = statsig_setup

    statsig_ai = StatsigAI.new_shared(
        statsig_source=StatsigCreateConfig(sdk_key="secret-key", statsig_options=statsig_options)
    )
    shared_statsig_ai = StatsigAI.shared()

    assert shared_statsig_ai == statsig_ai

    shared_statsig_ai.initialize()
    gate = shared_statsig_ai.get_statsig().check_gate(StatsigUser("my_user"), "test_public")
    print("hello i am gate here", gate)

    assert shared_statsig_ai.get_statsig().check_gate(StatsigUser("my_user"), "test_public")


def test_removing_shared_instance_with_create_config(statsig_setup):
    statsig_options, _ = statsig_setup

    statsig_ai = StatsigAI.new_shared(
        statsig_source=StatsigCreateConfig(sdk_key="secret-key", statsig_options=statsig_options)
    )
    statsig_ai.initialize()
    StatsigAI.remove_shared()

    shared_statsig_ai = StatsigAI.shared()
    assert not shared_statsig_ai.get_statsig().check_gate(StatsigUser("my_user"), "test_public")


def test_getting_shared_instance_with_attach_config(statsig_setup):
    statsig_options, _ = statsig_setup
    statsig = Statsig(sdk_key="secret-key", options=statsig_options)
    StatsigAI.new_shared(statsig_source=StatsigAttachConfig(statsig=statsig))
    shared_statsig_ai = StatsigAI.shared()
    # attach config do not manage the statsig instance lifecycle, so the statsig instance is not initialized
    assert not shared_statsig_ai.get_statsig().check_gate(StatsigUser("my_user"), "test_public")
    statsig.initialize().wait()
    shared_statsig_ai.initialize()

    assert shared_statsig_ai.get_statsig() == statsig
    assert shared_statsig_ai.get_statsig().check_gate(StatsigUser("my_user"), "test_public")


def test_removing_shared_instance_with_attach_config(statsig_setup):
    statsig_options, _ = statsig_setup
    statsig = Statsig(sdk_key="secret-key", options=statsig_options)
    statsig.initialize().wait()
    StatsigAI.new_shared(statsig_source=StatsigAttachConfig(statsig=statsig))
    StatsigAI.shared().initialize()
    StatsigAI.remove_shared()
    assert not StatsigAI.shared().get_statsig().check_gate(StatsigUser("my_user"), "test_public")



from typing import Any, Dict, Optional
from .configs import GenAICaptureOptions


def extract_genai_attributes(
    provider_name: str,
    operation_name: str,
    model: Any,
    kwargs: Dict[str, Any],
    response: Optional[Any] = None,
) -> Dict[str, Any]:
    attrs: Dict[str, Any] = {
        "gen_ai.provider.name": provider_name,
        "gen_ai.operation.name": operation_name,
        "gen_ai.request.model": model,
    }

    # ---------- Request attributes ----------
    attrs["gen_ai.request.max_tokens"] = kwargs.get("max_tokens") or kwargs.get(
        "max_completion_tokens"
    )
    attrs["gen_ai.request.temperature"] = kwargs.get("temperature")
    attrs["gen_ai.request.top_p"] = kwargs.get("top_p")
    attrs["gen_ai.request.top_k"] = kwargs.get("top_k")
    attrs["gen_ai.request.frequency_penalty"] = kwargs.get("frequency_penalty")
    attrs["gen_ai.request.presence_penalty"] = kwargs.get("presence_penalty")
    attrs["gen_ai.request.stop_sequences"] = kwargs.get("stop") or kwargs.get("stop_sequences")
    n = kwargs.get("n", 0)
    attrs["gen_ai.request.choice.count"] = n if n and n != 1 else None
    attrs["gen_ai.request.seed"] = kwargs.get("seed")
    attrs["gen_ai.conversation.id"] = kwargs.get("conversation_id")
    response_format = kwargs.get("response_format", {})
    attrs["gen_ai.output.type"] = response_format.get("type")

    # ---------- Response attributes ----------
    response = response or {}
    attrs["gen_ai.response.id"] = response.get("id", None)
    attrs["gen_ai.response.model"] = response.get("model", None)
    choices = response.get("choices", [])
    finish_reasons = [choice.get("finish_reason", "") for choice in choices]
    attrs["gen_ai.response.finish_reasons"] = finish_reasons

    # ---------- Usage attributes ----------
    usage = response.get("usage", {})
    attrs["gen_ai.usage.input_tokens"] = usage.get("prompt_tokens")
    attrs["gen_ai.usage.output_tokens"] = usage.get("completion_tokens")

    # ---------- Embeddings attributes ----------
    attrs.update(extract_embeddings_attributes(kwargs))

    # ---------- Provider-specific attributes ----------
    if provider_name == "openai":
        attrs.update(extract_openai_attributes(kwargs, response))

    # Missing attributes:
    # - service.address
    # - service.port

    return {k: v for k, v in attrs.items() if v is not None}


def extract_embeddings_attributes(kwargs: Dict[str, Any]) -> Dict[str, Any]:
    attrs: Dict[str, Any] = {}
    attrs["gen_ai.embeddings.dimension.count"] = kwargs.get("dimensions")
    attrs["gen_ai.request.encoding_formats"] = kwargs.get("encoding_format")
    return attrs


def extract_opt_in_attributes(
    gen_ai_capture_options: GenAICaptureOptions,
    kwargs: Dict[str, Any],
    response: Any,
) -> Dict[str, Any]:
    attrs: Dict[str, Any] = {}
    if gen_ai_capture_options.capture_input_messages or gen_ai_capture_options.capture_all:
        attrs["gen_ai.input.messages"] = kwargs.get("messages")
    if gen_ai_capture_options.capture_output_messages or gen_ai_capture_options.capture_all:
        attrs["gen_ai.output.messages"] = response.get("choices", [])
    if gen_ai_capture_options.capture_system_instructions or gen_ai_capture_options.capture_all:
        msgs = kwargs.get("messages") or []
        system_instructions = [msg for msg in msgs if msg.get("role") == "system"]
        attrs["gen_ai.system.instructions"] = system_instructions
    if gen_ai_capture_options.capture_tool_definitions or gen_ai_capture_options.capture_all:
        attrs["gen_ai.tool.definitions"] = kwargs.get("tools")
    return attrs


def extract_openai_attributes(kwargs: Dict[str, Any], response: Any) -> Dict[str, Any]:
    attrs: Dict[str, Any] = {}
    request_service_tier = kwargs.get("service_tier", "auto")
    attrs["openai.request.service_tier"] = (
        request_service_tier if request_service_tier != "auto" else None
    )
    attrs["openai.response.service_tier"] = response.get("service_tier")
    attrs["openai.response.system_fingerprint"] = response.get("system_fingerprint")
    return attrs



from typing import Any, Optional, Union

from statsig_python_core import Statsig, StatsigOptions
from .prompt import make_prompt, PromptEvaluationOptions, Prompt
from .ai_eval_grade_data import AIEvalGradeData
from .prompt_version import PromptVersion


class StatsigCreateConfig:
    def __init__(self, sdk_key: str, statsig_options: Optional[StatsigOptions] = None):
        self.sdk_key = sdk_key
        self.statsig_options = statsig_options


class StatsigAttachConfig:
    def __init__(self, statsig: Statsig):
        self.statsig = statsig


StatsigSourceConfig = Union[StatsigCreateConfig, StatsigAttachConfig]


class StatsigAIInstance:
    _statsig: Statsig
    _owns_statsig_instance: bool

    def __init__(
        self,
        statsig_source: StatsigSourceConfig,
    ):
        if isinstance(statsig_source, StatsigAttachConfig):
            self._statsig = statsig_source.statsig
            self._owns_statsig_instance = False
        else:
            self._statsig = Statsig(statsig_source.sdk_key, statsig_source.statsig_options)
            self._owns_statsig_instance = True

    def initialize(self) -> None:
        if self._owns_statsig_instance:
            self._statsig.initialize().wait()

    def flush_events(self) -> None:
        if self._owns_statsig_instance:
            self._statsig.flush_events().wait()

    def shutdown(self) -> None:
        if self._owns_statsig_instance:
            self._statsig.shutdown().wait()

    def get_statsig(self) -> Statsig:
        return self._statsig

    def get_prompt(
        self,
        user: Any,
        prompt_name: str,
        _options: Optional[PromptEvaluationOptions] = None,
    ) -> Prompt:
        MAX_DEPTH = 300
        depth = 0

        base_param_store_name = f"prompt:{prompt_name}"
        current_param_store_name = base_param_store_name
        next_param_store_name = (
            self._statsig.get_parameter_store(user, current_param_store_name).get_string(
                "prompt_targeting_rules", ""
            )
            or ""
        )

        while (
            next_param_store_name != ""
            and next_param_store_name != current_param_store_name
            and depth < MAX_DEPTH
        ):
            next_param_store = self._statsig.get_parameter_store(user, next_param_store_name)
            possible_next_param_store_name = (
                next_param_store.get_string("prompt_targeting_rules", "") or ""
            )
            if possible_next_param_store_name in [next_param_store_name, ""]:
                current_param_store_name = next_param_store_name
                break
            current_param_store_name = next_param_store_name
            next_param_store_name = possible_next_param_store_name
            depth += 1

        if depth >= MAX_DEPTH:
            current_param_store_name = base_param_store_name
            print(
                f"[Statsig] Max targeting depth ({MAX_DEPTH}) reached while resolving prompt: {prompt_name}. "
                + f'Possible circular reference starting from "{base_param_store_name}".'
            )
        final_param_store = self._statsig.get_parameter_store(user, current_param_store_name)
        prompt_name = current_param_store_name.split(":")[1]

        return make_prompt(self._statsig, prompt_name, final_param_store, user)

    def log_eval_grade(
        self,
        user: Any,
        prompt_version: PromptVersion,
        score: float,
        grader_name: str,
        eval_data: AIEvalGradeData,
    ) -> None:
        if score is None or score < 0 or score > 1:
            print(
                f"[Statsig] AI eval result score is out of bounds: {score} is not between 0 and 1, skipping log event"
            )
            return

        self._statsig.log_event(
            user,
            "statsig::eval_result",
            prompt_version.get_prompt_name(),
            {
                "score": str(score),
                "session_id": eval_data.get("session_id", ""),
                "version_name": prompt_version.get_name(),
                "version_id": prompt_version.get_id(),
                "grader_id": grader_name,
                "ai_config_name": prompt_version.get_prompt_name(),
            },
        )



from typing import List

import pytest

from opentelemetry import trace
from opentelemetry.sdk.trace import TracerProvider

from statsig_ai.otel.otel import InitializeTracingOptions, initialize_tracing


@pytest.fixture(autouse=True)
def set_env_and_reset_singleton(monkeypatch: pytest.MonkeyPatch):
    monkeypatch.setenv("STATSIG_SDK_KEY", "test-sdk-key")
    from statsig_ai.otel import singleton as singleton_module

    original_instance = getattr(singleton_module.OtelSingleton, "_instance", None)
    singleton_module.OtelSingleton._instance = None

    yield

    singleton_module.OtelSingleton._instance = original_instance


def test_initialize_tracing_components():
    result = initialize_tracing(
        InitializeTracingOptions(
            service_name="test-service",
            version="1.0.0",
            environment="test",
            skip_global_context_manager_setup=True,
        )
    )

    assert result.exporter is not None
    assert result.processor is not None
    assert result.provider is not None


def test_does_not_register_global_provider_by_default(monkeypatch: pytest.MonkeyPatch):
    calls: List[TracerProvider] = []

    def fake_set_tracer_provider(provider: TracerProvider):
        calls.append(provider)

    monkeypatch.setattr(trace, "set_tracer_provider", fake_set_tracer_provider)

    result = initialize_tracing(
        InitializeTracingOptions(
            service_name="test-service",
            version="1.0.0",
            environment="test",
            skip_global_context_manager_setup=True,
        )
    )

    assert result.provider is not None
    assert calls == []


def test_can_register_global_trace_provider(monkeypatch: pytest.MonkeyPatch):
    called_with: List[TracerProvider] = []

    def fake_set_tracer_provider(provider: TracerProvider):
        called_with.append(provider)

    monkeypatch.setattr(trace, "set_tracer_provider", fake_set_tracer_provider)

    initialize_tracing(
        InitializeTracingOptions(
            skip_global_context_manager_setup=True,
            enable_global_trace_provider_registration=True,
        )
    )

    assert len(called_with) == 1
    assert isinstance(called_with[0], TracerProvider)


def test_uses_provided_global_tracer_provider_for_singleton(monkeypatch: pytest.MonkeyPatch):
    from statsig_ai.otel.singleton import OtelSingleton

    custom_provider = TracerProvider()
    called = {"count": 0}

    def fake_set_tracer_provider(provider: TracerProvider):
        called["count"] += 1

    monkeypatch.setattr(trace, "set_tracer_provider", fake_set_tracer_provider)

    result = initialize_tracing(
        InitializeTracingOptions(
            global_trace_provider=custom_provider,
            skip_global_context_manager_setup=True,
        )
    )

    assert result.provider is not custom_provider
    assert OtelSingleton.get_instance().get_tracer_provider() is custom_provider
    assert called["count"] == 0


def test_populates_resource_attributes_from_options():
    result = initialize_tracing(
        InitializeTracingOptions(
            service_name="resource-service",
            version="2.0.0",
            environment="staging",
            skip_global_context_manager_setup=True,
        )
    )

    attrs = result.provider.resource.attributes  # type: ignore[attr-defined]

    assert attrs.get("service.name") == "resource-service"
    assert attrs.get("service.version") == "2.0.0"
    assert attrs.get("env") == "staging"


def test_falls_back_to_environment_variables_for_resource_attributes(
    monkeypatch: pytest.MonkeyPatch,
):
    monkeypatch.setenv("OTEL_SERVICE_NAME", "env-service-name")
    monkeypatch.setenv("PYTHON_ENV", "ci")

    result = initialize_tracing(
        InitializeTracingOptions(
            skip_global_context_manager_setup=True,
        )
    )

    attrs = result.provider.resource.attributes  # type: ignore[attr-defined]

    assert attrs.get("service.name") == "env-service-name"
    assert attrs.get("env") == "ci"



from typing import (
    Awaitable,
    Callable,
    Dict,
    Iterable,
    Iterator,
    Protocol,
    Sequence,
    Type,
    TypeVar,
    Generic,
    Optional,
    Union,
    Any,
    List,
)
from dataclasses import dataclass
from typing_extensions import TypedDict

from .serializable_data_class import SerializableDataClass

Input = TypeVar("Input")
Output = TypeVar("Output")


# ================================
# Eval Data Record Types
# ================================


@dataclass
class EvalDataRecord(SerializableDataClass, Generic[Input, Output]):
    input: Input
    category: Optional[Union[Sequence[str], str]] = None
    expected: Optional[Output] = None


class _EvalDataRecordDictVoidExpected(TypedDict, Generic[Input], total=False):
    input: Input
    category: Optional[Union[Sequence[str], str]]


class _EvalDataRecordDict(
    _EvalDataRecordDictVoidExpected[Input], Generic[Input, Output], total=False
):
    expected: Optional[Output]


_DataclassOrDictEvalDataRecord = Union[
    EvalDataRecord[Input, Output],
    _EvalDataRecordDict[Input, Output],
    _EvalDataRecordDictVoidExpected[Input],
]

_PossibleEvalDataRecords = Union[
    Iterable[_DataclassOrDictEvalDataRecord[Input, Output]],
    Iterator[_DataclassOrDictEvalDataRecord[Input, Output]],
    Awaitable[Iterator[_DataclassOrDictEvalDataRecord[Input, Output]]],
    Callable[
        [],
        Union[
            Iterable[_DataclassOrDictEvalDataRecord[Input, Output]],
            Awaitable[Iterable[_DataclassOrDictEvalDataRecord[Input, Output]]],
        ],
    ],
]

EvalData = Union[
    _PossibleEvalDataRecords[Input, Output], Type[_PossibleEvalDataRecords[Input, Output]]
]

# ================================
# Eval Result Types
# ================================


@dataclass
class EvalResultMetadata(SerializableDataClass):
    error: bool


@dataclass
class EvalResultRecord(SerializableDataClass, Generic[Input, Output]):
    input: Input
    output: Output
    scores: Dict[str, float]
    expected: Optional[Output] = None
    error: Optional[bool] = False
    category: Optional[Union[Sequence[str], str]] = None


@dataclass
class EvalResult(SerializableDataClass, Generic[Input, Output]):
    results: List[EvalResultRecord[Input, Output]]
    metadata: EvalResultMetadata
    summary_scores: Optional[Dict[str, float]] = None


# ================================
# Eval Scorer Type
# ================================

Score = Union[int, float, bool]


@dataclass
class EvalScorerArgs(SerializableDataClass, Generic[Input, Output]):
    input: Input
    output: Output
    expected: Optional[Output] = None


class SyncScorer(Protocol, Generic[Input, Output]):
    def __call__(self, args: EvalScorerArgs[Input, Output]) -> Score: ...


class AsyncScorer(Protocol, Generic[Input, Output]):
    async def score(self, args: EvalScorerArgs[Input, Output]) -> Score: ...


ScorerInterface = Union[SyncScorer[Input, Output], AsyncScorer[Input, Output]]

Scorer = Union[
    ScorerInterface[Input, Output],
    Type[ScorerInterface[Input, Output]],
    Callable[[EvalScorerArgs[Input, Output]], Score],
    Callable[[EvalScorerArgs[Input, Output]], Awaitable[Score]],
]

ScorerFnMap = Dict[str, Scorer[Input, Output]]

EvalScorer = Union[Scorer[Input, Output], ScorerFnMap[Input, Output]]


# ================================
# Eval Task Type
# ================================

EvalParameters = Dict[str, Any]


@dataclass
class EvalHook(SerializableDataClass):
    parameters: Optional[EvalParameters] = None
    category: Optional[Union[Sequence[str], str]] = None


EvalTask = Union[
    Callable[[Input], Union[Output, Awaitable[Output]]],
    Callable[[Input, EvalHook], Union[Output, Awaitable[Output]]],
]

SummaryScorerFn = Callable[[List[EvalResultRecord[Input, Output]]], Dict[str, float]]



from typing import Any, Optional
from .openai_impl import OpenAIWrapper
from .configs import WrapOpenAIOptions


def wrap_openai(oai_client: Any, options: Optional[WrapOpenAIOptions] = None):
    if options is None:
        options = WrapOpenAIOptions()
    return OpenAIWrapper(oai_client, options)



import os

from setuptools import setup, find_packages

with open(
    os.path.join(os.path.abspath(os.path.dirname(__file__)), "statsig_ai", "version.py"),
    encoding="utf-8",
) as f:
    exec(f.read())

with open(
    os.path.join(os.path.abspath(os.path.dirname(__file__)), "README.md"),
    encoding="utf-8",
) as r:
    README = r.read()

test_deps = [
    "pytest",
    "pytest-httpserver",
    "werkzeug",
    "statsig-python-core",
    "pytest-asyncio",
    "openai",
]
extras = {
    "test": test_deps,
}

setup(
    name="statsig_ai",
    # pylint: disable=undefined-variable
    version=__version__,  # type: ignore
    description="Statsig AI Python SDK",
    long_description=README,
    long_description_content_type="text/markdown",
    author="",
    author_email="",
    url="https://github.com/statsig-io/statsig-ai-python",
    license="ISC",
    classifiers=[
        "Intended Audience :: Developers",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    install_requires=[
        "requests",
        "statsig-python-core",
        "opentelemetry-api",
        "opentelemetry-sdk",
        "opentelemetry-exporter-otlp-proto-http",
        "typing-extensions>=4.1.0",
    ],
    tests_require=test_deps,
    extras_require=extras,
    include_package_data=True,
    packages=find_packages(include=["statsig_ai", "statsig_ai.*"]),
    python_requires=">=3.9",
)



from statsig_python_core import StatsigOptions, StatsigUser
from pytest_httpserver import HTTPServer
from statsig_ai import PromptVersion, StatsigAI, StatsigCreateConfig
from utils import get_test_data_resource
import pytest
from mock_scrapi import MockScrapi


@pytest.fixture
def statsig_setup(httpserver: HTTPServer):
    mock_scrapi = MockScrapi(httpserver)
    dcs_content = get_test_data_resource("eval_proj_dcs.json")

    mock_scrapi.stub(
        "/v2/download_config_specs/secret-key.json", response=dcs_content, method="GET"
    )
    mock_scrapi.stub("/v1/log_event", response='{"success": true}', method="POST")

    options = StatsigOptions(
        specs_url=mock_scrapi.url_for_endpoint("/v2/download_config_specs"),
        log_event_url=mock_scrapi.url_for_endpoint("/v1/log_event"),
    )

    yield options, mock_scrapi


def test_get_prompt(statsig_setup):
    options, _ = statsig_setup
    statsig_ai = StatsigAI(
        statsig_source=StatsigCreateConfig(sdk_key="secret-key", statsig_options=options)
    )
    statsig_ai.initialize()
    prompt = statsig_ai.get_prompt(StatsigUser("a-user"), "test_prompt")
    statsig_ai.shutdown()
    assert prompt.get_name() == "test_prompt"
    assert isinstance(prompt.get_live(), PromptVersion)
    assert isinstance(prompt.get_candicates(), list)
    assert all(isinstance(c, PromptVersion) for c in prompt.get_candicates())


def test_get_prompt_get_live(statsig_setup):
    options, _ = statsig_setup
    statsig_ai = StatsigAI(
        statsig_source=StatsigCreateConfig(sdk_key="secret-key", statsig_options=options)
    )
    statsig_ai.initialize()
    prompt = statsig_ai.get_prompt(StatsigUser("a-user"), "test_prompt")
    statsig_ai.shutdown()

    live_version = prompt.get_live()
    assert live_version is not None
    assert live_version.get_id() == "6KGzeo8TR9JTL7CZl7vccd"
    assert live_version.get_name() == "Version 1"
    assert live_version.get_temperature() == 1
    assert live_version.get_max_tokens() == 1000
    assert live_version.get_top_p() == 1
    assert live_version.get_frequency_penalty() == 0
    assert live_version.get_presence_penalty() == 0
    assert live_version.get_provider() == "openai"
    assert live_version.get_model() == "gpt-5"
    assert isinstance(live_version.get_workflow_body(), dict)
    assert live_version.get_eval_model() == "gpt-4o-mini"
    assert live_version.get_type() == "Live"
    assert live_version.get_prompt_name() == "test_prompt"


def test_get_prompt_get_candicates(statsig_setup):
    options, _ = statsig_setup
    statsig_ai = StatsigAI(
        statsig_source=StatsigCreateConfig(sdk_key="secret-key", statsig_options=options)
    )
    statsig_ai.initialize()
    prompt = statsig_ai.get_prompt(StatsigUser("a-user"), "test_prompt")
    statsig_ai.shutdown()

    candicates = prompt.get_candicates()
    assert len(candicates) == 2
    assert candicates[0].get_id() == "7jszgFEAi1KRA2Tot6qikg"
    assert candicates[0].get_name() == "Version 2"
    assert candicates[1].get_id() == "7CKLvQvOwjj2vjx12gFO0Z"
    assert candicates[1].get_name() == "Version 3"


def test_log_eval_grade(statsig_setup):
    options, mock_scrapi = statsig_setup
    statsig_ai = StatsigAI(
        statsig_source=StatsigCreateConfig(sdk_key="secret-key", statsig_options=options)
    )
    statsig_ai.initialize()
    prompt_version = statsig_ai.get_prompt(StatsigUser("a-user"), "test_prompt").get_live()
    statsig_ai.log_eval_grade(
        StatsigUser("a-user"), prompt_version, 0.5, "test_grader", {"session_id": "1234567890"}
    )
    statsig_ai.flush_events()

    logged_events = mock_scrapi.get_logged_events()
    eval_event = list(filter(lambda e: e["eventName"] == "statsig::eval_result", logged_events))
    assert len(eval_event) == 1
    assert eval_event[0]["eventName"] == "statsig::eval_result"
    assert eval_event[0]["user"]["userID"] == "a-user"
    assert eval_event[0]["value"] == "test_prompt"
    metadata = eval_event[0]["metadata"]
    assert metadata["score"] == "0.5"
    assert metadata["grader_id"] == "test_grader"
    assert metadata["ai_config_name"] == "test_prompt"
    assert metadata["version_name"] == "Version 1"
    assert metadata["version_id"] == "6KGzeo8TR9JTL7CZl7vccd"
    assert metadata["session_id"] == "1234567890"



from typing import List, TypedDict
from statsig_python_core import Statsig, StatsigUser
from .prompt_version import PromptVersion


class PromptEvaluationOptions(TypedDict):
    pass


class Prompt:
    def __init__(self, name: str, live: PromptVersion, candicates: List[PromptVersion]):
        self.name = name
        self.live = live
        self.candicates = candicates

    def get_name(self) -> str:
        return self.name

    def get_live(self) -> PromptVersion:
        return self.live

    def get_candicates(self) -> List[PromptVersion]:
        return self.candicates


def make_prompt(statsig: Statsig, name: str, paramStore, user: StatsigUser) -> Prompt:
    live_config_id = paramStore.get_string("live", "") or ""
    candidates_config_ids = paramStore.get_array("candidates", []) or []
    live_config = statsig.get_dynamic_config(user, live_config_id)
    candidates_configs = []
    for config_id in candidates_config_ids:
        config = statsig.get_dynamic_config(user, config_id)
        candidates_configs.append(config)
    return Prompt(
        name,
        PromptVersion(live_config),
        [PromptVersion(config) for config in candidates_configs],
    )
